@@ -54,7 +54,7 @@
 
 #include "libbalsa.h"
 #include "misc.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 #ifdef BALSA_USE_THREADS
 static pthread_t main_thread_id;
@@ -113,6 +113,9 @@ libbalsa_init(LibBalsaInformationFunc information_callback)
 #if HAVE_SQLITE
     LIBBALSA_TYPE_ADDRESS_BOOK_GPE;
 #endif
+#if HAVE_RUBRICA
+    LIBBALSA_TYPE_ADDRESS_BOOK_RUBRICA;
+#endif
 }
 
 
@@ -779,6 +782,7 @@ void (*libbalsa_progress_set_text) (LibBalsaProgress * progress,
                                     const gchar * text, guint total);
 void (*libbalsa_progress_set_fraction) (LibBalsaProgress * progress,
                                         gdouble fraction);
+void (*libbalsa_progress_set_activity) (gboolean set, const gchar * text);
 
 /*
  * Face and X-Face header support.
