@@ -52,6 +52,9 @@ typedef struct _LibBalsaCondition LibBalsaCondition;
 #if HAVE_SQLITE
 #include "address-book-gpe.h"
 #endif
+#if HAVE_RUBRICA
+#include "address-book-rubrica.h"
+#endif
 
 #include "mailbox.h"
 #include "mailbox_local.h"
@@ -151,15 +154,20 @@ typedef enum {
 #define LIBBALSA_PROGRESS_INIT LIBBALSA_PROGRESS_NO
 /* We will not use the progress bar if the number of increments is less
  * than LIBBALSA_PROGRESS_MIN_COUNT, and we will not update the fraction
- * if it has increased by less than LIBBALSA_PROGRESS_MIN_UPDATE. */
-#define LIBBALSA_PROGRESS_MIN_COUNT  400
-#define LIBBALSA_PROGRESS_MIN_UPDATE 0.02
+ * if the time since the last update is less than
+ * LIBBALSA_PROGRESS_MIN_UPDATE_SECS seconds or if the fraction has
+ * increased by less than LIBBALSA_PROGRESS_MIN_UPDATE_STEP. */
+#define LIBBALSA_PROGRESS_MIN_COUNT        400
+#define LIBBALSA_PROGRESS_MIN_UPDATE_USECS 50000
+#define LIBBALSA_PROGRESS_MIN_UPDATE_STEP  0.05
 
 extern void (*libbalsa_progress_set_text) (LibBalsaProgress * progress,
                                            const gchar * text,
                                            guint total);
 extern void (*libbalsa_progress_set_fraction) (LibBalsaProgress * progress,
                                                gdouble fraction);
+extern void (*libbalsa_progress_set_activity) (gboolean set,
+                                               const gchar * text);
 
 /*
  * Face and X-Face header support.
