@@ -34,7 +34,7 @@
 #include "libbalsa_private.h"
 #include "libbalsa-marshal.h"
 #include "libbalsa-conf.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 static GObjectClass *parent_class = NULL;
 static void libbalsa_server_class_init(LibBalsaServerClass * klass);
@@ -388,8 +388,10 @@ libbalsa_server_user_cb(ImapUserEventType ue, void *arg, ...)
         ok = va_arg(alist, int*);
         vfy_result = va_arg(alist, long);
         reason =  X509_verify_cert_error_string(vfy_result);
+#if 0
         printf("IMAP:TLS: failed cert verification: %ld : %s.\n",
                vfy_result, reason);
+#endif
         ssl = va_arg(alist, SSL*);
         cert = SSL_get_peer_certificate(ssl);
 	if(cert) {
