@@ -39,11 +39,12 @@
 #include "balsa-app.h"
 #include "save-restore.h"
 
-#include "i18n.h"	/* Must come after balsa-app.h. */
+#include <glib/gi18n.h>	/* Must come after balsa-app.h. */
 
 /* Global application structure */
 struct BalsaApplication balsa_app;
 
+#if !HAVE_GTKSPELL
 const gchar *pspell_modules[] = {
     "ispell",
     "aspell"
@@ -54,6 +55,7 @@ const gchar *pspell_suggest_modes[] = {
     "normal",
     "bad-spellers"
 };
+#endif                          /* HAVE_GTKSPELL */
 
 /* ask_password:
    asks the user for the password to the mailbox on given remote server.
@@ -296,8 +298,12 @@ balsa_app_init(void)
     balsa_app.mblist_width = 100;
     balsa_app.mw_width = MW_DEFAULT_WIDTH;
     balsa_app.mw_height = MW_DEFAULT_HEIGHT;
+    balsa_app.mw_maximized = FALSE;
+
     balsa_app.sw_width = 0;
     balsa_app.sw_height = 0;
+    balsa_app.sw_maximized = FALSE;
+
     balsa_app.toolbar_wrap_button_text = TRUE;
     balsa_app.pwindow_option = WHILERETR;
     balsa_app.wordwrap = TRUE;
@@ -385,6 +391,7 @@ balsa_app_init(void)
     /* spell check */
 #if HAVE_GTKSPELL
     balsa_app.spell_check_lang = NULL;
+    balsa_app.spell_check_active = FALSE;
 #else                           /* HAVE_GTKSPELL */
     balsa_app.module = SPELL_CHECK_MODULE_ASPELL;
     balsa_app.suggestion_mode = SPELL_CHECK_SUGGEST_NORMAL;
@@ -456,7 +463,7 @@ balsa_app_destroy(void)
 static gint
 check_new_messages_auto_cb(gpointer data)
 {
-    check_new_messages_real((GtkWidget *) NULL, data, TYPE_BACKGROUND);
+    check_new_messages_real(data, TYPE_BACKGROUND);
 
     if (balsa_app.debug)
         fprintf(stderr, "Auto-checked for new messages...\n");
