@@ -225,11 +225,14 @@ extern struct BalsaApplication {
     /* GUI settings */
     gint mw_width;
     gint mw_height;
+    gboolean mw_maximized;
     gint mblist_width;
     gint sw_width; /* sendmsg window */
     gint sw_height;
+    gboolean sw_maximized;
     gint message_window_width;
     gint message_window_height;
+    gboolean message_window_maximized;
 
     /* toolbars */
     int toolbar_wrap_button_text;
@@ -364,8 +367,7 @@ extern struct BalsaApplication {
     gboolean always_queue_sent_mail;
     gboolean copy_to_sentbox;
 
-    /* appbar */
-    GnomeAppBar *appbar;
+    /* mailbox indices */
     GtkWidget *notebook;
 
     /* address book */
@@ -374,7 +376,8 @@ extern struct BalsaApplication {
 
     /* spell checking */
 #if HAVE_GTKSPELL
-    gchar *spell_check_lang;
+    gchar   *spell_check_lang;
+    gboolean spell_check_active;
 #else                           /* HAVE_GTKSPELL */
     SpellCheckModule module;
     SpellCheckSuggestMode suggestion_mode;
