@@ -49,7 +49,7 @@
 
 #include "filter-funcs.h"
 #include "misc.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 /* TREE_VIEW_FIXED_HEIGHT enables hight-performance mode of GtkTreeView
  * very useful for large mailboxes (#msg >5000) but: a. is available only
@@ -138,7 +138,7 @@ static void bndx_expand_to_row(BalsaIndex * index, GtkTreePath * path);
 static void bndx_select_row(BalsaIndex * index, GtkTreePath * path);
 
 /* Other callbacks. */
-static void bndx_store_address(GtkWidget * widget, gpointer data);
+static void bndx_store_address(gpointer data);
 
 static GtkTreeViewClass *parent_class = NULL;
 
@@ -454,21 +454,26 @@ bndx_selection_changed_real(BalsaIndex * index)
 
     if (msgno) {
         GtkTreePath *path;
-        index->current_message_is_deleted =
-            libbalsa_mailbox_msgno_has_flags(mailbox, msgno,
-                                             LIBBALSA_MESSAGE_FLAG_DELETED,
-                                             0);
-        libbalsa_mailbox_msgno_change_flags(mailbox, msgno,
-                                            LIBBALSA_MESSAGE_FLAG_SELECTED,
-                                            0);
-        if (libbalsa_mailbox_msgno_find(mailbox, msgno, &path, NULL)) {
+
+        if (!libbalsa_mailbox_msgno_find(mailbox, msgno, &path, NULL))
+            msgno = 0;
+        else {
             GtkTreeSelection *selection =
                 gtk_tree_view_get_selection(GTK_TREE_VIEW(index));
+
             if (!gtk_tree_selection_path_is_selected(selection, path)) {
                 bndx_expand_to_row(index, path);
                 bndx_select_row(index, path);
             }
             gtk_tree_path_free(path);
+
+            index->current_message_is_deleted =
+                libbalsa_mailbox_msgno_has_flags(mailbox, msgno,
+                                                 LIBBALSA_MESSAGE_FLAG_DELETED,
+                                                 0);
+            libbalsa_mailbox_msgno_change_flags(mailbox, msgno,
+                                                LIBBALSA_MESSAGE_FLAG_SELECTED,
+                                                0);
         }
     }
 
@@ -601,8 +606,7 @@ bndx_row_activated(GtkTreeView * tree_view, GtkTreePath * path,
          * instead we'll just use the guts of
          * balsa_message_continue: */
         BalsaSendmsg *sm =
-            sendmsg_window_continue(GTK_WIDGET(BALSA_INDEX(tree_view)->window),
-                                    mailbox, msgno);
+            sendmsg_window_continue(mailbox, msgno);
         g_signal_connect(G_OBJECT(sm->window), "destroy",
                          G_CALLBACK(sendmsg_window_destroy_cb), NULL);
     } else
@@ -620,14 +624,16 @@ bndx_scroll_idle(BalsaIndex * index)
     GtkTreePath *path;
 
     gdk_threads_enter();
-
-    row_ref = g_object_get_data(G_OBJECT(index), BALSA_INDEX_ROW_REF_KEY);
-    if (row_ref && (path = gtk_tree_row_reference_get_path(row_ref))) {
-        gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(index), path, NULL,
-                                     FALSE, 0, 0);
-        gtk_tree_path_free(path);
+    /* The index might have been destroyed since the idle function was
+       registered. Check whether we still exist... */
+    if(gtk_tree_view_get_model(GTK_TREE_VIEW(index))) {
+        row_ref = g_object_get_data(G_OBJECT(index), BALSA_INDEX_ROW_REF_KEY);
+        if (row_ref && (path = gtk_tree_row_reference_get_path(row_ref))) {
+            gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(index), path, NULL,
+                                         FALSE, 0, 0);
+            gtk_tree_path_free(path);
+        }
     }
-
     g_object_set_data(G_OBJECT(index), BALSA_INDEX_ROW_REF_KEY, NULL);
     g_object_unref(index);
 
@@ -644,13 +650,15 @@ bndx_scroll_to_row(BalsaIndex * index, GtkTreePath * path)
     row_ref = g_object_get_data(G_OBJECT(index), BALSA_INDEX_ROW_REF_KEY);
     if (!row_ref) {
         g_object_ref(index);
-        g_idle_add((GSourceFunc) bndx_scroll_idle, index);
+        g_idle_add_full(G_PRIORITY_LOW, (GSourceFunc) bndx_scroll_idle,
+                        index, NULL);
     }
 
     row_ref =
         gtk_tree_row_reference_new(gtk_tree_view_get_model
                                    (GTK_TREE_VIEW(index)), path);
-    g_object_set_data_full(G_OBJECT(index), BALSA_INDEX_ROW_REF_KEY, row_ref,
+    g_object_set_data_full(G_OBJECT(index), BALSA_INDEX_ROW_REF_KEY,
+                           row_ref,
                            (GDestroyNotify) gtk_tree_row_reference_free);
 }
 
@@ -766,7 +774,8 @@ balsa_index_new(void)
  * the index, or the last message if none is unread, and selects
  * it. Since this routine is usually called from a thread, we have to
  * take care and and make sure the row is selected from the main
- * thread only.
+ * thread only. And we also check whether the mailbox hasn't been
+ * destroyed by now, of course.
  */
 struct view_on_open_data {
     BalsaIndex *bindex;
@@ -776,8 +785,10 @@ static gboolean
 bi_view_on_open(struct view_on_open_data *data)
 {
     gdk_threads_enter();
-    bndx_select_row(data->bindex, data->path);
-    gtk_tree_path_free(data->path);
+    if(gtk_tree_view_get_model(GTK_TREE_VIEW(data->bindex))) {
+        bndx_select_row(data->bindex, data->path);
+        gtk_tree_path_free(data->path);
+    }
     g_object_unref(data->bindex);
     g_free(data);
     gdk_threads_leave();
@@ -894,6 +905,21 @@ bndx_mailbox_row_inserted_cb(LibBalsaMailbox * mailbox, GtkTreePath * path,
     }
 }
 
+static void
+bndx_mailbox_message_expunged_cb(LibBalsaMailbox * mailbox, guint msgno,
+                                 BalsaIndex * bindex)
+{
+    if (bindex->current_msgno == msgno)
+        bindex->current_msgno = 0;
+    else if (bindex->current_msgno > msgno)
+        --bindex->current_msgno;
+
+    if (bindex->next_msgno == msgno)
+        bindex->next_msgno = 0;
+    else if (bindex->next_msgno > msgno)
+        --bindex->next_msgno;
+}
+
 /* balsa_index_load_mailbox_node:
    open mailbox_node, the opening is done in thread to keep UI alive.
 
@@ -907,10 +933,8 @@ balsa_index_load_mailbox_node (BalsaIndex * index,
 {
     GtkTreeView *tree_view;
     LibBalsaMailbox* mailbox;
-    gchar *msg;
     gboolean successp;
     gint try_cnt;
-    LibBalsaCondition *view_filter;
 
     g_return_val_if_fail(BALSA_IS_INDEX(index), TRUE);
     g_return_val_if_fail(index->mailbox_node == NULL, TRUE);
@@ -919,13 +943,6 @@ balsa_index_load_mailbox_node (BalsaIndex * index,
 
     mailbox = mbnode->mailbox;
 
-    msg = g_strdup_printf(_("Opening mailbox %s. Please wait..."),
-			  mbnode->mailbox->name);
-    gdk_threads_enter();
-    gnome_appbar_push(balsa_app.appbar, msg);
-    gdk_threads_leave();
-    g_free(msg);
-
     try_cnt = 0;
     do {
         g_clear_error(err);
@@ -939,10 +956,6 @@ balsa_index_load_mailbox_node (BalsaIndex * index,
         balsa_mblist_close_lru_peer_mbx(balsa_app.mblist, mailbox);
     } while(try_cnt++<3);
 
-    gdk_threads_enter();
-    gnome_appbar_pop(balsa_app.appbar);
-    gdk_threads_leave();
-
     if (!successp)
 	return TRUE;
 
@@ -969,22 +982,9 @@ balsa_index_load_mailbox_node (BalsaIndex * index,
 			     (gpointer) index);
     g_signal_connect(mailbox, "row-inserted",
 	    	     G_CALLBACK(bndx_mailbox_row_inserted_cb), index);
+    g_signal_connect(mailbox, "message-expunged",
+	    	     G_CALLBACK(bndx_mailbox_message_expunged_cb), index);
 
-    libbalsa_mailbox_register_msgno(mailbox, &index->current_msgno);
-    libbalsa_mailbox_register_msgno(mailbox, &index->next_msgno);
-
-    balsa_window_enable_mailbox_menus(balsa_app.main_window, index);
-    gdk_threads_leave();
-
-    view_filter = balsa_window_get_view_filter(balsa_app.main_window, TRUE);
-    libbalsa_mailbox_set_view_filter(mailbox, view_filter, FALSE);
-    libbalsa_condition_unref(view_filter);
-    libbalsa_mailbox_make_view_filter_persistent(mailbox);
-    libbalsa_mailbox_set_threading(mailbox,
-                                   libbalsa_mailbox_get_threading_type
-                                   (mailbox));
-
-    gdk_threads_enter();
     /* Set the tree store*/
 #ifndef GTK2_FETCHES_ONLY_VISIBLE_CELLS
     g_object_set_data(G_OBJECT(mailbox), "tree-view", tree_view);
@@ -1327,18 +1327,6 @@ bndx_mailbox_changed_func(BalsaIndex * bindex)
     LibBalsaMailbox *mailbox = bindex->mailbox_node->mailbox;
     GtkTreePath *path;
 
-    if (!GTK_WIDGET_REALIZED(GTK_WIDGET(bindex)))
-        return;
-
-    if (bindex->current_msgno) {
-        if (libbalsa_mailbox_msgno_has_flags(mailbox,
-                                             bindex->current_msgno, 0,
-                                             LIBBALSA_MESSAGE_FLAG_DELETED))
-            bindex->current_message_is_deleted = FALSE;
-        else if (!bindex->current_message_is_deleted)
-            bndx_select_next_threaded(bindex);
-    }
-
     if (mailbox->first_unread
         && libbalsa_mailbox_msgno_find(mailbox, mailbox->first_unread,
                                        &path, NULL)) {
@@ -1381,8 +1369,23 @@ bndx_mailbox_changed_idle(struct index_info* arg)
 static void
 bndx_mailbox_changed_cb(BalsaIndex * bindex)
 {
+    LibBalsaMailbox *mailbox = bindex->mailbox_node->mailbox;
     struct index_info *arg;
 
+    if (!GTK_WIDGET_REALIZED(GTK_WIDGET(bindex)))
+        return;
+
+    /* Find the next message to be shown now, not later in the idle
+     * callback. */
+    if (bindex->current_msgno) {
+        if (libbalsa_mailbox_msgno_has_flags(mailbox,
+                                             bindex->current_msgno, 0,
+                                             LIBBALSA_MESSAGE_FLAG_DELETED))
+            bindex->current_message_is_deleted = FALSE;
+        else if (!bindex->current_message_is_deleted)
+            bndx_select_next_threaded(bindex);
+    }
+
     if (bindex->has_mailbox_changed_idle)
         return;
 
@@ -1426,7 +1429,7 @@ balsa_index_selected_msgnos_free(BalsaIndex * index, GArray * msgnos)
 }
 
 static void
-bndx_view_source(GtkWidget * widget, gpointer data)
+bndx_view_source(gpointer data)
 {
     BalsaIndex *index = BALSA_INDEX(data);
     LibBalsaMailbox *mailbox = index->mailbox_node->mailbox;
@@ -1438,6 +1441,8 @@ bndx_view_source(GtkWidget * widget, gpointer data)
         LibBalsaMessage *message =
             libbalsa_mailbox_get_message(mailbox, msgno);
 
+        if (!message)
+            continue;
 	libbalsa_show_message_source(message, balsa_app.message_font,
 				     &balsa_app.source_escape_specials);
         g_object_unref(message);
@@ -1446,7 +1451,7 @@ bndx_view_source(GtkWidget * widget, gpointer data)
 }
 
 static void
-bndx_store_address(GtkWidget * widget, gpointer data)
+bndx_store_address(gpointer data)
 {
     GList *messages = balsa_index_selected_list(BALSA_INDEX(data));
 
@@ -1465,6 +1470,8 @@ balsa_index_selected_list_func(GtkTreeModel * model, GtkTreePath * path,
 
     gtk_tree_model_get(model, iter, LB_MBOX_MSGNO_COL, &msgno, -1);
     message = libbalsa_mailbox_get_message(LIBBALSA_MAILBOX(model), msgno);
+    if (!message)
+        return;
     *list = g_list_prepend(*list, message);
 }
 
@@ -1492,8 +1499,7 @@ balsa_index_selected_list(BalsaIndex * index)
  * message
  */
 static void
-bndx_compose_foreach(GtkWidget * w, BalsaIndex * index,
-                     SendType send_type)
+bndx_compose_foreach(BalsaIndex * index, SendType send_type)
 {
     LibBalsaMailbox *mailbox = index->mailbox_node->mailbox;
     GArray *selected = balsa_index_selected_msgnos_new(index);
@@ -1506,10 +1512,10 @@ bndx_compose_foreach(GtkWidget * w, BalsaIndex * index,
         case SEND_REPLY:
         case SEND_REPLY_ALL:
         case SEND_REPLY_GROUP:
-            sm = sendmsg_window_reply(NULL, mailbox, msgno, send_type);
+            sm = sendmsg_window_reply(mailbox, msgno, send_type);
             break;
         case SEND_CONTINUE:
-            sm = sendmsg_window_continue(NULL, mailbox, msgno);
+            sm = sendmsg_window_continue(mailbox, msgno);
             break;
         default:
             g_assert_not_reached();
@@ -1525,27 +1531,27 @@ bndx_compose_foreach(GtkWidget * w, BalsaIndex * index,
  * Public `reply' methods
  */
 void
-balsa_message_reply(GtkWidget * widget, gpointer user_data)
+balsa_message_reply(gpointer user_data)
 {
-    bndx_compose_foreach(widget, BALSA_INDEX (user_data), SEND_REPLY);
+    bndx_compose_foreach(BALSA_INDEX (user_data), SEND_REPLY);
 }
 
 void
-balsa_message_replytoall(GtkWidget * widget, gpointer user_data)
+balsa_message_replytoall(gpointer user_data)
 {
-    bndx_compose_foreach(widget, BALSA_INDEX (user_data), SEND_REPLY_ALL);
+    bndx_compose_foreach(BALSA_INDEX (user_data), SEND_REPLY_ALL);
 }
 
 void
-balsa_message_replytogroup(GtkWidget * widget, gpointer user_data)
+balsa_message_replytogroup(gpointer user_data)
 {
-    bndx_compose_foreach(widget, BALSA_INDEX (user_data), SEND_REPLY_GROUP);
+    bndx_compose_foreach(BALSA_INDEX (user_data), SEND_REPLY_GROUP);
 }
 
 void
-balsa_message_continue(GtkWidget * widget, gpointer user_data)
+balsa_message_continue(gpointer user_data)
 {
-    bndx_compose_foreach(widget, BALSA_INDEX (user_data), SEND_CONTINUE);
+    bndx_compose_foreach(BALSA_INDEX (user_data), SEND_CONTINUE);
 }
 
 /*
@@ -1553,12 +1559,11 @@ balsa_message_continue(GtkWidget * widget, gpointer user_data)
  * selected messages
  */
 static void
-bndx_compose_from_list(GtkWidget * w, BalsaIndex * index,
-                       SendType send_type)
+bndx_compose_from_list(BalsaIndex * index, SendType send_type)
 {
     GArray *selected = balsa_index_selected_msgnos_new(index);
     BalsaSendmsg *sm =
-        sendmsg_window_new_from_list(w, index->mailbox_node->mailbox,
+        sendmsg_window_new_from_list(index->mailbox_node->mailbox,
                                      selected, send_type);
 
     balsa_index_selected_msgnos_free(index, selected);
@@ -1570,25 +1575,23 @@ bndx_compose_from_list(GtkWidget * w, BalsaIndex * index,
  * Public forwarding methods
  */
 void
-balsa_message_forward_attached(GtkWidget * widget, gpointer user_data)
+balsa_message_forward_attached(gpointer user_data)
 {
-    bndx_compose_from_list(widget, BALSA_INDEX(user_data),
-                           SEND_FORWARD_ATTACH);
+    bndx_compose_from_list(BALSA_INDEX(user_data), SEND_FORWARD_ATTACH);
 }
 
 void
-balsa_message_forward_inline(GtkWidget * widget, gpointer user_data)
+balsa_message_forward_inline(gpointer user_data)
 {
-    bndx_compose_from_list(widget, BALSA_INDEX(user_data),
-                           SEND_FORWARD_INLINE);
+    bndx_compose_from_list(BALSA_INDEX(user_data), SEND_FORWARD_INLINE);
 }
 
 void
-balsa_message_forward_default(GtkWidget * widget, gpointer user_data)
+balsa_message_forward_default(gpointer user_data)
 {
-    bndx_compose_from_list(widget, BALSA_INDEX(user_data),
-                           balsa_app.forward_attached
-                           ? SEND_FORWARD_ATTACH : SEND_FORWARD_INLINE);
+    bndx_compose_from_list(BALSA_INDEX(user_data),
+                           balsa_app.forward_attached ?
+                           SEND_FORWARD_ATTACH : SEND_FORWARD_INLINE);
 }
 
 /*
@@ -1639,7 +1642,7 @@ bndx_do_delete(BalsaIndex* index, gboolean move_to_trash)
  * Public message delete methods
  */
 void
-balsa_message_move_to_trash(GtkWidget * widget, gpointer user_data)
+balsa_message_move_to_trash(gpointer user_data)
 {
     BalsaIndex *index;
 
@@ -1707,7 +1710,7 @@ balsa_index_toggle_flag(BalsaIndex* index, LibBalsaMessageFlag flag)
 }
 
 static void
-bi_toggle_deleted_cb(GtkWidget * widget, gpointer user_data)
+bi_toggle_deleted_cb(gpointer user_data, GtkWidget * widget)
 {
     BalsaIndex *index;
     GArray *selected;
@@ -1733,7 +1736,7 @@ bi_toggle_deleted_cb(GtkWidget * widget, gpointer user_data)
 /* This function toggles the FLAGGED attribute of a list of messages
  */
 static void
-bi_toggle_flagged_cb(GtkWidget * widget, gpointer user_data)
+bi_toggle_flagged_cb(gpointer user_data)
 {
     g_return_if_fail(user_data != NULL);
 
@@ -1742,7 +1745,7 @@ bi_toggle_flagged_cb(GtkWidget * widget, gpointer user_data)
 }
 
 static void
-bi_toggle_new_cb(GtkWidget * widget, gpointer user_data)
+bi_toggle_new_cb(gpointer user_data)
 {
     g_return_if_fail(user_data != NULL);
 
@@ -1768,12 +1771,6 @@ mru_menu_cb(gchar * url, BalsaIndex * index)
 /*
  * bndx_popup_menu_create: create the popup menu at init time
  */
-static void
-bndx_pipe_message(GtkWidget *widget, gpointer user_data)
-{
-    balsa_index_pipe(BALSA_INDEX(user_data));
-}
-
 static GtkWidget *
 bndx_popup_menu_create(BalsaIndex * index)
 {
@@ -1793,7 +1790,7 @@ bndx_popup_menu_create(BalsaIndex * index)
         BALSA_PIXMAP_FORWARD, N_("Forward _Inline..."),
                 GTK_SIGNAL_FUNC(balsa_message_forward_inline)}, {
         NULL,                 N_("_Pipe through..."),
-                GTK_SIGNAL_FUNC(bndx_pipe_message)}, {
+                GTK_SIGNAL_FUNC(balsa_index_pipe)}, {
         BALSA_PIXMAP_BOOK_RED, N_("_Store Address..."),
                 GTK_SIGNAL_FUNC(bndx_store_address)}};
     GtkWidget *menu, *menuitem, *submenu;
@@ -1905,10 +1902,11 @@ bndx_do_popup(BalsaIndex * index, GdkEventButton * event)
     gtk_widget_set_sensitive(index->move_to_item,
                              any && !mailbox->readonly);
 
-    submenu = balsa_mblist_mru_menu(GTK_WINDOW(index->window),
-                                    &balsa_app.folder_mru,
-                                    G_CALLBACK(mru_menu_cb),
-                                    index);
+    submenu =
+        balsa_mblist_mru_menu(GTK_WINDOW
+                              (gtk_widget_get_toplevel(GTK_WIDGET(index))),
+                              &balsa_app.folder_mru,
+                              G_CALLBACK(mru_menu_cb), index);
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(index->move_to_item),
                               submenu);
 
@@ -1935,7 +1933,8 @@ create_stock_menu_item(GtkWidget * menu, const gchar * type,
 
     gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), image);
 
-    g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(cb), data);
+    g_signal_connect_swapped(G_OBJECT(menuitem), "activate",
+                             G_CALLBACK(cb), data);
 
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
@@ -2376,7 +2375,10 @@ balsa_index_ensure_visible(BalsaIndex * index)
     GdkRectangle rect;
     GtkTreePath *path = NULL;
 
-    if (bndx_find_current_msgno(index, &path, NULL)) {
+    if (!GTK_WIDGET_REALIZED(tree_view))
+        return;
+
+    if (!bndx_find_current_msgno(index, &path, NULL)) {
         /* Current message not displayed, make sure that something
            else is... */
         gtk_tree_view_get_visible_rect(tree_view, &rect);
