@@ -31,7 +31,7 @@
 #include "balsa-index.h"
 #include "balsa-mblist.h"
 #include "libbalsa.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "mailbox-node.h"
 #include "main-window.h"
 
@@ -1074,10 +1074,6 @@ bmbl_mailbox_changed_cb(LibBalsaMailbox * mailbox, gpointer data)
 
     g_return_if_fail(LIBBALSA_IS_MAILBOX(mailbox));
 
-    /* Signal may be emitted without holding the gdk lock, so we'd
-     * better grab it. */
-    gdk_threads_enter();
-
     umd = g_object_get_data(G_OBJECT(mailbox), "mblist-update");
 
     if (!umd) {
@@ -1091,8 +1087,6 @@ bmbl_mailbox_changed_cb(LibBalsaMailbox * mailbox, gpointer data)
 
     umd->notify = (mailbox->state == LB_MAILBOX_STATE_OPEN
                    || mailbox->state == LB_MAILBOX_STATE_CLOSED);
-
-    gdk_threads_leave();
 }
 
 /* public methods */
@@ -2294,6 +2288,8 @@ balsa_mblist_set_status_bar(LibBalsaMailbox * mailbox)
     gint unread_messages = mailbox->unread_messages;
     gint hidden_messages;
     GString *desc = g_string_new(NULL);
+    GtkStatusbar *statusbar;
+    guint context_id;
 
     hidden_messages =
         mailbox->msg_tree ? total_messages -
@@ -2325,7 +2321,10 @@ balsa_mblist_set_status_bar(LibBalsaMailbox * mailbox)
                                    hidden_messages);
     }
 
-    gnome_appbar_set_default(balsa_app.appbar, desc->str);
+    statusbar = GTK_STATUSBAR(balsa_app.main_window->statusbar);
+    context_id = gtk_statusbar_get_context_id(statusbar, "BalsaMBList message");
+    gtk_statusbar_pop(statusbar, context_id);
+    gtk_statusbar_push(statusbar, context_id, desc->str);
 
     g_string_free(desc, TRUE);
 }
