@@ -21,14 +21,9 @@
 
 #include "config.h"
 
-#if defined(BALSA_USE_THREADS) && defined(THREADED_IMAP_SCAN_FIXED)
-#include <pthread.h>
-/* for sched_yield() prototype */
-#include <sched.h>
-#endif
-
 #include <unistd.h>
 #include <string.h>
+
 #include "balsa-app.h"
 #include "folder-scanners.h"
 #include "folder-conf.h"
@@ -37,6 +32,8 @@
 #include "save-restore.h"
 #include "filter.h"
 #include "imap-server.h"
+#include <glib/gi18n.h>
+#include "libbalsa-conf.h"
 
 /* MailboxNode object is a GUI representation of a mailbox, or entire 
    set of them. It can read itself from the configuration, save its data,
@@ -60,16 +57,17 @@ static void balsa_mailbox_node_dispose(GObject * object);
 static void balsa_mailbox_node_finalize(GObject * object);
 
 static void balsa_mailbox_node_real_save_config(BalsaMailboxNode* mn,
-						const gchar * prefix);
+						const gchar * group);
 static void balsa_mailbox_node_real_load_config(BalsaMailboxNode* mn,
-						const gchar * prefix);
+						const gchar * group);
 
 static BalsaMailboxNode *imap_scan_create_mbnode(BalsaMailboxNode * root,
 						 imap_scan_item * isi,
-						 char delim);
+						 int delim);
 static gboolean imap_scan_attach_mailbox(BalsaMailboxNode * mbnode,
                                          imap_scan_item * isi);
 static gboolean bmbn_scan_children_idle(BalsaMailboxNode ** mn);
+static gboolean restore_children_from_cache(BalsaMailboxNode *mn);
 
 static BalsaMailboxNode *add_local_mailbox(BalsaMailboxNode * root,
 					   const gchar * name,
@@ -83,8 +81,7 @@ static void handle_imap_path(const char *fn, char delim, int noselect,
 			     int noscan, int marked, void *data);
 static gint check_imap_path(const char *fn, LibBalsaServer * server,
 			    guint depth);
-static void mark_imap_path(const gchar * fn, gint noselect, gint noscan,
-			   gpointer data);
+static void mark_imap_path(const gchar * fn, gpointer data);
 
 enum {
     SAVE_CONFIG,
@@ -212,6 +209,7 @@ balsa_mailbox_node_finalize(GObject * object)
 
     mn = BALSA_MAILBOX_NODE(object);
 
+    balsa_mailbox_node_clear_children_cache(mn);
     mn->parent  = NULL; 
     g_free(mn->name);          mn->name = NULL;
     g_free(mn->dir);           mn->dir = NULL;
@@ -228,27 +226,77 @@ balsa_mailbox_node_finalize(GObject * object)
 }
 
 static void
-balsa_mailbox_node_real_save_config(BalsaMailboxNode* mn, const gchar * prefix)
+balsa_mailboxes_append_children(GtkTreeModel *model, GtkTreeIter *parent,
+                                GPtrArray *children_names)
 {
-    if(mn->name)
-	printf("Saving mailbox node %s with prefix %s\n", mn->name, prefix);
-    libbalsa_imap_server_save_config(LIBBALSA_IMAP_SERVER(mn->server));
-    gnome_config_set_string("Name",      mn->name);
-    gnome_config_set_string("Directory", mn->dir);
-    gnome_config_set_bool("Subscribed",  mn->subscribed);
-    gnome_config_set_bool("ListInbox",   mn->list_inbox);
-    
-    g_free(mn->config_prefix);
-    mn->config_prefix = g_strdup(prefix);
+    GtkTreeIter child;
+
+    if (!gtk_tree_model_iter_children(model, &child, parent))
+	return;
+    do {
+        BalsaMailboxNode *mbnode;
+        gchar *str;
+
+        gtk_tree_model_get(model, &child, 0, &mbnode, -1);
+        g_return_if_fail(mbnode->server);
+        if(mbnode->mailbox) {
+            const char *path = 
+                libbalsa_mailbox_imap_get_path
+                (LIBBALSA_MAILBOX_IMAP(mbnode->mailbox));
+
+            str = g_strconcat("PXS", path, NULL);
+        } else {
+            str = g_strconcat("PXD", mbnode->dir, NULL);
+        }
+        str[1] = mbnode->delim;
+        g_ptr_array_add(children_names, str);
+
+        g_object_unref(mbnode);
+
+        balsa_mailboxes_append_children(model, &child, children_names);
+    } while(gtk_tree_model_iter_next(model, &child));
 }
 
 static void
-balsa_mailbox_node_real_load_config(BalsaMailboxNode* mn, const gchar * prefix)
+balsa_mailbox_node_real_save_config(BalsaMailboxNode* mn, const gchar * group)
+{
+    GPtrArray *children_names;
+    GtkTreeIter iter;
+    
+    g_return_if_fail(!mn->parent);
+
+    if(mn->name)
+	printf("Saving mailbox node %s with group %s\n", mn->name, group);
+    libbalsa_imap_server_save_config(LIBBALSA_IMAP_SERVER(mn->server));
+    libbalsa_conf_set_string("Name",      mn->name);
+    libbalsa_conf_set_string("Directory", mn->dir);
+    libbalsa_conf_set_bool("Subscribed",  mn->subscribed);
+    libbalsa_conf_set_bool("ListInbox",   mn->list_inbox);
+    
+    if (balsa_find_iter_by_data(&iter, mn)) {
+        GtkTreeModel *model = GTK_TREE_MODEL(balsa_app.mblist_tree_store);
+
+        children_names = g_ptr_array_new();
+        balsa_mailboxes_append_children(model, &iter,
+                                        children_names);
+        printf("Saving %d children\n", children_names->len);
+        libbalsa_conf_set_vector("Children", children_names->len,
+                                 (const char*const*)(children_names->pdata));
+        g_ptr_array_foreach(children_names, (GFunc)g_free, NULL);
+        g_ptr_array_free(children_names, TRUE);
+    }
+
+    g_free(mn->config_prefix);
+    mn->config_prefix = g_strdup(group);
+}
+
+static void
+balsa_mailbox_node_real_load_config(BalsaMailboxNode* mn, const gchar * group)
 {
     g_free(mn->config_prefix);
-    mn->config_prefix = g_strdup(prefix);
+    mn->config_prefix = g_strdup(group);
     g_free(mn->name);
-    mn->name = gnome_config_get_string("Name");
+    mn->name = libbalsa_conf_get_string("Name");
 }
 
 BalsaMailboxNode *
@@ -319,6 +367,8 @@ mark_local_path(BalsaMailboxNode *mbnode)
     mbnode->scanned = TRUE;
 }
 
+/** Read local directory in search for mailboxes. Never does any
+    caching - it is not worth it. */
 static void
 read_dir_cb(BalsaMailboxNode* mb)
 {
@@ -382,6 +432,8 @@ imap_scan_attach_mailbox(BalsaMailboxNode * mbnode, imap_scan_item * isi)
 					 (gpointer) isi->special);
         *isi->special = LIBBALSA_MAILBOX(m);
 	g_object_add_weak_pointer(G_OBJECT(m), (gpointer) isi->special);
+        if (isi->special == &balsa_app.outbox)
+            LIBBALSA_MAILBOX(m)->no_reassemble = TRUE;
     }
 
     return TRUE;
@@ -408,19 +460,36 @@ struct imap_scan_tree_ {
 
 static void imap_scan_destroy_tree(imap_scan_tree * tree);
 
-static void*
-imap_dir_cb_real(void* r)
+static void
+imap_dir_cb(BalsaMailboxNode* mb)
 {
+    gchar* msg;
+    BalsaMailboxNode* mroot=mb;
     BalsaMailboxNode *n;
     GSList *list;
-    BalsaMailboxNode*mb = r;
     GError *error = NULL;
     imap_scan_tree imap_tree = { NULL, '.' };
+    GtkStatusbar *statusbar;
+    guint context_id;
 
-    g_return_val_if_fail(mb->server, NULL);
+    if (!balsa_app.main_window)
+        return;
 
-    libbalsa_scanner_imap_dir(mb, mb->server, mb->dir, mb->subscribed,
-                              mb->list_inbox, 
+    statusbar = GTK_STATUSBAR(balsa_app.main_window->statusbar);
+    context_id = gtk_statusbar_get_context_id(statusbar, "MailboxNode imap");
+
+    if(restore_children_from_cache(mb))
+        return;
+
+    while(mroot->parent)
+	mroot = mroot->parent;
+    msg = g_strdup_printf(_("Scanning %s. Please wait..."), mroot->name);
+    gtk_statusbar_push(statusbar, context_id, msg);
+    g_free(msg);
+
+    g_return_if_fail(mb->server);
+    libbalsa_scanner_imap_dir(mb, mb->server, mb->dir, mb->delim,
+			      mb->subscribed, mb->list_inbox, 
                               check_imap_path,
                               mark_imap_path,
                               handle_imap_path,
@@ -428,7 +497,7 @@ imap_dir_cb_real(void* r)
                               &error);
     
     if(error) {
-        libbalsa_information(LIBBALSA_INFORMATION_ERROR,
+        libbalsa_information(LIBBALSA_INFORMATION_WARNING,
                              error->code == LIBBALSA_MAILBOX_NETWORK_ERROR
                              ? _("Scanning of %s failed: %s\n"
                                  "Check network connectivity.")
@@ -437,14 +506,14 @@ imap_dir_cb_real(void* r)
                              error->message);
         g_error_free(error);
         imap_scan_destroy_tree(&imap_tree);
-        gnome_appbar_pop(balsa_app.appbar);
-        return NULL;
+        gtk_statusbar_pop(statusbar, context_id);
+        return;
     }
 
     if (!balsa_app.mblist_tree_store) {
         /* Quitt'n time! */
         imap_scan_destroy_tree(&imap_tree);
-        return NULL;
+        return;
     }
 
     /* phase b. */
@@ -466,30 +535,11 @@ imap_dir_cb_real(void* r)
     if(balsa_app.debug && mb->name)
         printf("imap_dir_cb:  main mailbox node %s mailbox is %p\n", 
                mb->name, mb->mailbox);
-    gnome_appbar_pop(balsa_app.appbar);
     if(balsa_app.debug) printf("%d: Scanning done.\n", (int)time(NULL));
-    return NULL;
-}
+    gtk_statusbar_pop(statusbar, context_id);
 
-static void
-imap_dir_cb(BalsaMailboxNode* mb)
-{
-    gchar* msg;
-    BalsaMailboxNode* mroot=mb;
-#if defined(BALSA_USE_THREADS) && defined(THREADED_IMAP_SCAN_FIXED)
-    pthread_t scan_th_id;
-#endif
-    while(mroot->parent)
-	mroot = mroot->parent;
-    msg = g_strdup_printf(_("Scanning %s. Please wait..."), mroot->name);
-    gnome_appbar_push(balsa_app.appbar, msg);
-    g_free(msg);
-#if defined(BALSA_USE_THREADS) && defined(THREADED_IMAP_SCAN_FIXED)
-    pthread_create(&scan_th_id, NULL, imap_dir_cb_real, mb);
-    pthread_detach(scan_th_id);
-#else
-    imap_dir_cb_real(mb);
-#endif
+    /* We can save the cache now... */
+    config_folder_update(mroot);
 }
 
 BalsaMailboxNode *
@@ -535,30 +585,43 @@ balsa_mailbox_node_new_from_dir(const gchar* dir)
    similiar to mailbox creation from configuration data.
 */
 BalsaMailboxNode*
-balsa_mailbox_node_new_from_config(const gchar* prefix)
+balsa_mailbox_node_new_from_config(const gchar* group)
 {
+    gboolean def;
+    gint n_children;
+    gchar **children;
     BalsaMailboxNode * folder = balsa_mailbox_node_new();
-    gnome_config_push_prefix(prefix);
+
+    libbalsa_conf_push_group(group);
 
     folder->server = LIBBALSA_SERVER(libbalsa_imap_server_new_from_config());
 
     if(balsa_app.debug)
 	printf("Server loaded, host: %s, %s.\n", folder->server->host,
 	       folder->server->use_ssl ? "SSL" : "no SSL");
+    g_signal_connect_swapped(G_OBJECT(folder->server), "config-changed", 
+                             G_CALLBACK(config_folder_update), folder);
     g_signal_connect(G_OBJECT(folder), "show-prop-dialog", 
 		     G_CALLBACK(folder_conf_imap_node), NULL);
     g_signal_connect(G_OBJECT(folder), "append-subtree", 
 		     G_CALLBACK(imap_dir_cb), NULL);
     libbalsa_server_connect_signals(folder->server,
                                     G_CALLBACK(ask_password), NULL);
-    balsa_mailbox_node_load_config(folder, prefix);
+    balsa_mailbox_node_load_config(folder, group);
 
-    folder->dir = gnome_config_get_string("Directory");
+    folder->dir = libbalsa_conf_get_string("Directory");
     folder->subscribed =
-	gnome_config_get_bool("Subscribed"); 
+	libbalsa_conf_get_bool("Subscribed"); 
     folder->list_inbox =
-	gnome_config_get_bool("ListInbox=true"); 
-    gnome_config_pop_prefix();
+	libbalsa_conf_get_bool("ListInbox=true"); 
+
+    libbalsa_conf_get_vector_with_default("Children",&n_children,
+                                          &children,&def);
+    if(!def) {
+        g_object_set_data(G_OBJECT(folder), "children-cache", children);
+    }
+
+    libbalsa_conf_pop_group();
 
     return folder;
 }
@@ -630,17 +693,17 @@ balsa_mailbox_node_show_prop_dialog_cb(GtkWidget * widget, gpointer data)
    with some sane defaults.
 */
 void
-balsa_mailbox_node_load_config(BalsaMailboxNode* mn, const gchar* prefix)
+balsa_mailbox_node_load_config(BalsaMailboxNode* mn, const gchar* group)
 {
     g_signal_emit(G_OBJECT(mn),
-		  balsa_mailbox_node_signals[LOAD_CONFIG], 0, prefix);
+		  balsa_mailbox_node_signals[LOAD_CONFIG], 0, group);
 }
 
 void
-balsa_mailbox_node_save_config(BalsaMailboxNode* mn, const gchar* prefix)
+balsa_mailbox_node_save_config(BalsaMailboxNode* mn, const gchar* group)
 {
     g_signal_emit(G_OBJECT(mn),
-		  balsa_mailbox_node_signals[SAVE_CONFIG], 0, prefix);
+		  balsa_mailbox_node_signals[SAVE_CONFIG], 0, group);
 }
 
 /* ---------------------------------------------------------------------
@@ -683,33 +746,56 @@ balsa_mailbox_local_append(LibBalsaMailbox* mbx)
 void
 balsa_mailbox_node_rescan(BalsaMailboxNode * mn)
 {
-#if defined(BALSA_USE_THREADS) && defined(THREADED_IMAP_SCAN_FIXED)
-    GNode *gnode;
-
-    if (!balsa_app.mailbox_nodes)
-        return;
-
-    gnode = balsa_find_mbnode(balsa_app.mailbox_nodes, mn);
-
-    if(gnode) {
-	balsa_remove_children_mailbox_nodes(gnode);
-	balsa_mailbox_node_append_subtree(mn, gnode);
-        mn->scanned = TRUE;
-    } else {
-        g_warning("folder node %s (%p) not found in hierarchy.\n",
-		     mn->name, mn);
-    }
-#else
-
     if (!balsa_app.mblist_tree_store)
         return;
 
+    balsa_mailbox_node_clear_children_cache(mn);
     balsa_remove_children_mailbox_nodes(mn);
     balsa_mailbox_node_append_subtree(mn ? mn : balsa_app.root_node);
-
-#endif  /* defined(BALSA_USE_THREADS) && defined(THREADED_IMAP_SCAN_FIXED) */
 }
 
+void
+balsa_mailbox_node_clear_children_cache(BalsaMailboxNode *mn)
+{
+    gchar **children_cache = g_object_get_data(G_OBJECT(mn), "children-cache");
+
+    if(children_cache) {
+        g_strfreev(children_cache);
+        g_object_set_data(G_OBJECT(mn), "children-cache", NULL);
+    }
+}
+
+static gboolean
+restore_children_from_cache(BalsaMailboxNode *mn)
+{
+    gint i;
+    gchar **children = g_object_get_data(G_OBJECT(mn), "children-cache");
+    imap_scan_item isi;
+
+    if(!children) {
+        if(mn->name)
+            printf("No cache for %s - quitting.\n", mn->name);
+        return FALSE;
+    }
+
+    for(i=0; children[i]; i++) {
+        if(children[i][0] == 'P' && strlen(children[i])>3) {
+            BalsaMailboxNode *n;
+            isi.fn = children[i]+3;
+            isi.special = NULL;
+            isi.scanned = TRUE;
+            isi.selectable = children[i][2] == 'S';
+            isi.marked = FALSE;
+            n = imap_scan_create_mbnode(mn, &isi, children[i][1]);
+            if (isi.selectable && imap_scan_attach_mailbox(n, &isi))
+                balsa_mblist_mailbox_node_redraw(n);
+        }
+    }
+    
+    return TRUE;
+}
+
+    
 /* balsa_mailbox_node_scan_children:
  * checks whether a mailbox node's children need scanning. 
  * Note that rescanning local_mail_directory will *not* trigger rescanning
@@ -769,19 +855,25 @@ bmbn_scan_children_idle(BalsaMailboxNode ** mbnode)
     list = g_object_get_data(G_OBJECT(*mbnode), BALSA_MAILBOX_NODE_LIST_KEY);
     for (l = list; l; l = g_slist_next(l)) {
         BalsaMailboxNode *mn;
-        gboolean has_unread_messages = FALSE;
         
         if (!l->data)
             continue;
         mn = l->data;
-        if (mn->mailbox)
-            has_unread_messages = mn->mailbox->has_unread_messages;
-        balsa_mailbox_node_rescan(mn);
-        if (!l->data)
-            continue;
-        if (mn->mailbox)
-            mn->mailbox->has_unread_messages = has_unread_messages;
-        mn->scanned = TRUE;
+        if (!mn->scanned) {
+            gboolean has_unread_messages = FALSE;
+
+            if (mn->mailbox)
+                has_unread_messages = mn->mailbox->has_unread_messages;
+            balsa_mailbox_node_rescan(mn);
+            if (!l->data)
+                continue;
+            if (mn->mailbox)
+                mn->mailbox->has_unread_messages = has_unread_messages;
+            mn->scanned = TRUE;
+        } else if (balsa_app.debug)
+            g_print("%s: %s \"%s\" was already scanned\n", __func__,
+                    mn->mailbox ? "mailbox" : "folder",
+                    mn->mailbox ? mn->mailbox->name : mn->name);
         g_object_remove_weak_pointer(G_OBJECT(mn), & l->data);
     }
     g_slist_free(list);
@@ -909,12 +1001,20 @@ mb_filter_cb(GtkWidget * widget, BalsaMailboxNode * mbnode)
 	g_print("You can apply filters only on mailbox\n");
 }
 
+static void
+mb_empty_trash_cb(GtkWidget * widget, BalsaMailboxNode * mbnode)
+{
+    empty_trash(balsa_app.main_window);
+}
+
 GtkWidget *
 balsa_mailbox_node_get_context_menu(BalsaMailboxNode * mbnode)
 {
     GtkWidget *menu;
     GtkWidget *submenu;
     GtkWidget *menuitem;
+    LibBalsaMailbox *mailbox;
+    gboolean special;
 
     /*  g_return_val_if_fail(mailbox != NULL, NULL); */
 
@@ -951,57 +1051,66 @@ balsa_mailbox_node_get_context_menu(BalsaMailboxNode * mbnode)
 	return menu;
     }
     /* If we didn't click on a mailbox node then there is only one option. */
-    add_menu_entry(menu, NULL, NULL, mbnode);
+    add_menu_entry(menu, NULL, NULL, NULL);
 
-    if (mbnode->mailbox) {
-	if (!MAILBOX_OPEN(mbnode->mailbox))
-	    add_menu_entry(menu, _("_Open"),
-                           G_CALLBACK(mb_open_cb), mbnode);
-	else
-	    add_menu_entry(menu, _("_Close"),
-                           G_CALLBACK(mb_close_cb), mbnode);
-    }
     if (g_signal_has_handler_pending(G_OBJECT(mbnode),
                                      balsa_mailbox_node_signals
                                      [SHOW_PROP_DIALOG], 0, FALSE))
         add_menu_entry(menu, _("_Properties..."),
                        G_CALLBACK(mb_conf_cb), mbnode);
-    if (mbnode->mailbox || mbnode->config_prefix)
-	add_menu_entry(menu, _("_Delete"), 
-                       G_CALLBACK(mb_del_cb),  mbnode);
+
     if (g_signal_has_handler_pending(G_OBJECT(mbnode),
 		                     balsa_mailbox_node_signals
 				     [APPEND_SUBTREE], 0, FALSE))
 	add_menu_entry(menu, _("_Rescan"),
 		       GTK_SIGNAL_FUNC(mb_rescan_cb), mbnode);
 
-    
-    if (mbnode->mailbox) {
-	add_menu_entry(menu, NULL, NULL, mbnode);
-	if(LIBBALSA_IS_MAILBOX_IMAP(mbnode->mailbox)) {
-	    add_menu_entry(menu, _("_Subscribe"),   
-                           GTK_SIGNAL_FUNC(mb_subscribe_cb),   mbnode);
-	    add_menu_entry(menu, _("_Unsubscribe"), 
-                           GTK_SIGNAL_FUNC(mb_unsubscribe_cb), mbnode);
-	}
-	
-	if(mbnode->mailbox != balsa_app.inbox)
-	    add_menu_entry(menu, _("Mark as _Inbox"),    
-                           GTK_SIGNAL_FUNC(mb_inbox_cb),    mbnode);
-	if(mbnode->mailbox != balsa_app.sentbox)
-	    add_menu_entry(menu, _("_Mark as Sentbox"), 
-                           GTK_SIGNAL_FUNC(mb_sentbox_cb),  mbnode);
-	if(mbnode->mailbox != balsa_app.trash)
-	    add_menu_entry(menu, _("Mark as _Trash"),    
-                           GTK_SIGNAL_FUNC(mb_trash_cb),    mbnode);
-	if(mbnode->mailbox != balsa_app.draftbox)
-	    add_menu_entry(menu, _("Mark as D_raftbox"),
-                           GTK_SIGNAL_FUNC(mb_draftbox_cb), mbnode);
-	/* FIXME : No test on mailbox type is made yet, should we ? */
-	add_menu_entry(menu, _("_Edit/Apply filters"), 
-                       GTK_SIGNAL_FUNC(mb_filter_cb), mbnode);
+    if (mbnode->config_prefix)
+	add_menu_entry(menu, _("_Delete"), G_CALLBACK(mb_del_cb),  mbnode);
+
+    if (!(mailbox = mbnode->mailbox))
+        return menu;
+
+    if (!MAILBOX_OPEN(mailbox))
+        add_menu_entry(menu, _("_Open"),  G_CALLBACK(mb_open_cb),  mbnode);
+    else
+        add_menu_entry(menu, _("_Close"), G_CALLBACK(mb_close_cb), mbnode);
+
+    special = (   mailbox == balsa_app.inbox
+               || mailbox == balsa_app.sentbox
+               || mailbox == balsa_app.draftbox
+               || mailbox == balsa_app.outbox
+               || mailbox == balsa_app.trash);
+    if (!special && !mbnode->config_prefix)
+	add_menu_entry(menu, _("_Delete"), G_CALLBACK(mb_del_cb),  mbnode);
+
+    if(LIBBALSA_IS_MAILBOX_IMAP(mailbox)) {
+        add_menu_entry(menu, NULL, NULL, NULL);
+        add_menu_entry(menu, _("_Subscribe"),   
+                       GTK_SIGNAL_FUNC(mb_subscribe_cb),   mbnode);
+        add_menu_entry(menu, _("_Unsubscribe"), 
+                       GTK_SIGNAL_FUNC(mb_unsubscribe_cb), mbnode);
     }
 
+    if (!special) {
+        add_menu_entry(menu, NULL, NULL, NULL);
+        add_menu_entry(menu, _("Mark as _Inbox"),    
+                       GTK_SIGNAL_FUNC(mb_inbox_cb),    mbnode);
+        add_menu_entry(menu, _("_Mark as Sentbox"), 
+                       GTK_SIGNAL_FUNC(mb_sentbox_cb),  mbnode);
+        add_menu_entry(menu, _("Mark as _Trash"),    
+                       GTK_SIGNAL_FUNC(mb_trash_cb),    mbnode);
+        add_menu_entry(menu, _("Mark as D_raftbox"),
+                       GTK_SIGNAL_FUNC(mb_draftbox_cb), mbnode);
+    } else if (mailbox == balsa_app.trash)
+        add_menu_entry(menu, _("_Empty trash"),    
+                       GTK_SIGNAL_FUNC(mb_empty_trash_cb), mbnode);
+
+    /* FIXME : No test on mailbox type is made yet, should we ? */
+    add_menu_entry(menu, NULL, NULL, NULL);
+    add_menu_entry(menu, _("_Edit/Apply filters"), 
+                   GTK_SIGNAL_FUNC(mb_filter_cb), mbnode);
+
     return menu;
 }
 
@@ -1056,6 +1165,14 @@ remove_special_mailbox_by_url(const gchar* url, LibBalsaMailbox *** special)
     return mailbox ? remove_mailbox_from_nodes(*mailbox) : NULL;
 }
 
+static gboolean
+mailbox_check_idle(LibBalsaMailbox * mailbox)
+{
+    libbalsa_mailbox_check(mailbox);
+    g_object_unref(mailbox);
+    return FALSE;
+}
+
 static BalsaMailboxNode *
 add_local_mailbox(BalsaMailboxNode *root, const gchar * name,
 		  const gchar * path, GType type)
@@ -1095,8 +1212,10 @@ add_local_mailbox(BalsaMailboxNode *root, const gchar * name,
 	    g_print(_("Local mailbox %s loaded as: %s\n"),
 		    mailbox->name,
 		    g_type_name(G_OBJECT_TYPE(mailbox)));
-	if (balsa_app.check_mail_upon_startup)
-	    libbalsa_mailbox_check(mailbox);
+	if (balsa_app.check_mail_upon_startup) {
+            g_object_ref(mailbox);
+            g_idle_add((GSourceFunc) mailbox_check_idle, mailbox);
+        }
     }
     g_free(url);
     /* no type checking, parent is NULL for root */
@@ -1145,7 +1264,7 @@ get_parent_folder_name(const gchar* path, char delim)
  */
 static BalsaMailboxNode *
 imap_scan_create_mbnode(BalsaMailboxNode * root, imap_scan_item * isi,
-			char delim)
+			int delim)
 {
     gchar *parent_name;
     BalsaMailboxNode *mbnode;
@@ -1159,15 +1278,14 @@ imap_scan_create_mbnode(BalsaMailboxNode * root, imap_scan_item * isi,
 	/* A mailbox with this url is already in the tree... */
 	BalsaMailboxNode *special =
 	    remove_special_mailbox_by_url(url, &isi->special);
-	if (!special) {
-	    /* ...and it's not special, so we'll return this mbnode. */
-	    g_free(url);
-	    return mbnode;
-	}
+	if (special) {
+            mailbox = special->mailbox;
+            g_object_ref(mailbox);
+            g_object_unref(special);
+	} else {
+            balsa_mblist_mailbox_node_remove(mbnode);
+        }
 	g_object_unref(mbnode);
-	mailbox = special->mailbox;
-	g_object_ref(mailbox);
-	g_object_unref(special);
     }
     g_free(url);
 
@@ -1187,6 +1305,7 @@ imap_scan_create_mbnode(BalsaMailboxNode * root, imap_scan_item * isi,
     else
         basename++;
     mbnode->name = g_strdup(basename);
+    mbnode->delim = delim;
     mbnode->parent = parent;
     mbnode->subscribed = parent->subscribed;
     mbnode->scanned = isi->scanned;
@@ -1284,15 +1403,10 @@ check_imap_path(const gchar *fn, LibBalsaServer * server, guint depth)
 
 /* mark_imap_path:
  *
- * find the imap_scan_item for fn and set flags.
- *
- * noselect may be -1, 0, or 1; -1 means no change, 0 or 1
- * mean set accordingly.
- *
- * noscan is always set
+ * find the imap_scan_item for fn and set scanned.
  */
 static void
-mark_imap_path(const gchar * fn, gint noselect, gint noscan, gpointer data)
+mark_imap_path(const gchar * fn, gpointer data)
 {
     imap_scan_tree *tree = data;
     GSList *list;
@@ -1302,11 +1416,7 @@ mark_imap_path(const gchar * fn, gint noselect, gint noscan, gpointer data)
     for (list = tree->list; list; list = list->next) {
         imap_scan_item *item = list->data;
         if (!strcmp(item->fn, fn)) {
-            if (noselect >= 0)
-        	item->selectable = !noselect;
-            item->scanned = noscan;
-            if (balsa_app.debug)
-        	printf(" noselect %d, noscan %d.\n", noselect, noscan);
+            item->scanned = TRUE;
             break;
         }
     }
