@@ -38,11 +38,18 @@ enum MailboxCheckType {
 
 typedef struct _BalsaWindow BalsaWindow;
 typedef struct _BalsaWindowClass BalsaWindowClass;
+typedef enum {
+    BALSA_PROGRESS_NONE = 0,
+    BALSA_PROGRESS_ACTIVITY,
+    BALSA_PROGRESS_INCREMENT
+} BalsaWindowProgress;
+
 
 struct _BalsaWindow {
-    GnomeApp window;
+    GtkWindow window;
 
     GtkWidget *progress_bar;
+    GtkWidget *statusbar;
     GtkWidget *mblist;
     GtkWidget *sos_entry;       /* SenderOrSubject filter entry */
     GtkWidget *notebook;
@@ -51,12 +58,25 @@ struct _BalsaWindow {
     GtkWidget *vpaned;
     GtkWidget *current_index;
     GtkWidget *filter_choice;
+    GtkWidget *vbox;
+    GtkWidget *content;
 
     guint set_message_id;
+
+    GtkActionGroup *action_group;
+    GtkActionGroup *mailbox_action_group;
+    GtkActionGroup *message_action_group;
+    GtkActionGroup *modify_message_action_group;
+
+    /* Progress bar stuff: */
+    BalsaWindowProgress progress_type;
+    guint activity_handler;
+    guint activity_counter;
+    GSList *activity_messages;
 };
 
 struct _BalsaWindowClass {
-    GnomeAppClass parent_class;
+    GtkWindowClass parent_class;
 
     void (*open_mbnode)  (BalsaWindow * window, BalsaMailboxNode * mbnode);
     void (*close_mbnode) (BalsaWindow * window, BalsaMailboxNode * mbnode);
@@ -80,11 +100,7 @@ enum {
 GtkType balsa_window_get_type(void);
 GtkWidget *balsa_window_new(void);
 GtkWidget *balsa_window_find_current_index(BalsaWindow * window);
-void balsa_window_enable_mailbox_menus(BalsaWindow * window,
-				       BalsaIndex * index);
 void balsa_window_update_book_menus(BalsaWindow *window);
-LibBalsaCondition* balsa_window_get_view_filter(BalsaWindow *window,
-                                                gboolean flags_only);
 void balsa_window_refresh(BalsaWindow * window);
 void balsa_window_open_mbnode(BalsaWindow * window, BalsaMailboxNode*mbnode);
 void balsa_window_close_mbnode(BalsaWindow * window, BalsaMailboxNode*mbnode);
@@ -94,25 +110,32 @@ void balsa_window_update_tab(BalsaMailboxNode * mbnode);
 void enable_empty_trash(BalsaWindow * window, TrashState status);
 void balsa_window_enable_continue(BalsaWindow * window);
 void balsa_change_window_layout(BalsaWindow *window);
-gboolean mail_progress_notify_cb(void);
-gboolean send_progress_notify_cb(void);
-void check_new_messages_cb(GtkWidget *, gpointer data);
-void check_new_messages_real(GtkWidget *, gpointer data, int type);
+gboolean mail_progress_notify_cb(GIOChannel * source,
+                                 GIOCondition condition,
+                                 BalsaWindow ** window);
+gboolean send_progress_notify_cb(GIOChannel * source,
+                                 GIOCondition condition,
+                                 BalsaWindow ** window);
+void check_new_messages_cb(GtkAction * action, gpointer data);
+void check_new_messages_real(BalsaWindow * window, int type);
 void check_new_messages_count(LibBalsaMailbox * mailbox, gboolean notify);
 void empty_trash(BalsaWindow * window);
 void update_view_menu(BalsaWindow * window);
-BalsaToolbarModel *balsa_window_get_toolbar_model(void);
+BalsaToolbarModel *balsa_window_get_toolbar_model(GtkUIManager **
+                                                  ui_manager);
 void balsa_window_select_all(GtkWindow * window);
 gboolean balsa_window_next_unread(BalsaWindow * window);
 
 /* functions to manipulate the progress bars of the window */
-void balsa_window_increase_activity(BalsaWindow* window);
-void balsa_window_decrease_activity(BalsaWindow* window);
 gboolean balsa_window_setup_progress(BalsaWindow * window,
                                      const gchar * text);
 void balsa_window_clear_progress(BalsaWindow* window);
 void balsa_window_increment_progress(BalsaWindow * window,
                                      gdouble fraction, gboolean flush);
+void balsa_window_increase_activity(BalsaWindow * window,
+                                    const gchar * message);
+void balsa_window_decrease_activity(BalsaWindow * window,
+                                    const gchar * message);
 
 #if defined(__FILE__) && defined(__LINE__)
 # ifdef __FUNCTION__
