@@ -39,9 +39,11 @@
 #include <locale.h>
 #endif
 
-#include "i18n.h"
+#ifdef HAVE_RUBRICA
+#include <libxml/xmlversion.h>
+#endif
 
-#include "address-entry.h"
+#include <glib/gi18n.h>
 
 #include "balsa-app.h"
 #include "balsa-icons.h"
@@ -60,6 +62,7 @@
 #include "Balsa.h"
 #include "balsa-bonobo.h"
 #include <bonobo-activation/bonobo-activation.h>
+#include <bonobo/bonobo-exception.h>
 
 #ifdef HAVE_GPGME
 #include <string.h>
@@ -111,6 +114,23 @@ static GSList* opt_attach_list = NULL;
 /* opt_compose_email: To: field for the compose window */
 static gchar *opt_compose_email = NULL;
 
+static void
+accel_map_load(void)
+{
+    gchar *accel_map_filename =
+        g_build_filename(g_get_home_dir(), ".balsa", "accelmap", NULL);
+    gtk_accel_map_load(accel_map_filename);
+    g_free(accel_map_filename);
+}
+
+static void
+accel_map_save(void)
+{
+    gchar *accel_map_filename =
+        g_build_filename(g_get_home_dir(), ".balsa", "accelmap", NULL);
+    gtk_accel_map_save(accel_map_filename);
+    g_free(accel_map_filename);
+}
 
 static void
 balsa_handle_automation_options() {
@@ -125,10 +145,11 @@ balsa_handle_automation_options() {
        ("OAFIID:GNOME_Balsa_Application_Factory",
 	Bonobo_ACTIVATION_FLAG_EXISTING_ONLY,
 	NULL, &ev);
-
-   if (factory) {
+   
+   if ( !(BONOBO_EX (&ev) || factory == CORBA_OBJECT_NIL) ) {
        /* there already is a server. good */
        CORBA_Object app;
+       printf("Another Balsa found. Talking to it...\n");
        app =  
 	   bonobo_activation_activate_from_id ("OAFIID:GNOME_Balsa_Application",
 						   0, NULL, &ev);
@@ -412,7 +433,8 @@ threads_init(void)
     mail_thread_msg_receive =
 	g_io_channel_unix_new(mail_thread_pipes[0]);
     g_io_add_watch(mail_thread_msg_receive, G_IO_IN,
-		   (GIOFunc) mail_progress_notify_cb, NULL);
+		   (GIOFunc) mail_progress_notify_cb, 
+                   &balsa_app.main_window);
     
     if (pipe(send_thread_pipes) < 0) {
 	g_log("BALSA Init", G_LOG_LEVEL_DEBUG,
@@ -422,7 +444,8 @@ threads_init(void)
     send_thread_msg_receive =
 	g_io_channel_unix_new(send_thread_pipes[0]);
     g_io_add_watch(send_thread_msg_receive, G_IO_IN,
-		   (GIOFunc) send_progress_notify_cb, NULL);
+		   (GIOFunc) send_progress_notify_cb,
+                   &balsa_app.main_window);
 }
 
 static void
@@ -602,7 +625,8 @@ periodic_expunge_cb(void)
 /*
  * Initialize the progress bar and set text.
  */
-static gdouble prev_fraction;
+static GTimeVal prev_time_val;
+static gdouble  min_fraction;
 static void
 balsa_progress_set_text(LibBalsaProgress * progress, const gchar * text,
                         guint total)
@@ -618,7 +642,8 @@ balsa_progress_set_text(LibBalsaProgress * progress, const gchar * text,
 
     if (!text || total >= LIBBALSA_PROGRESS_MIN_COUNT)
         rc = balsa_window_setup_progress(balsa_app.main_window, text);
-    prev_fraction = 0;
+    g_get_current_time(&prev_time_val);
+    min_fraction = LIBBALSA_PROGRESS_MIN_UPDATE_STEP;
     gdk_threads_leave();
 
     *progress = (text && rc) ?
@@ -632,11 +657,24 @@ balsa_progress_set_text(LibBalsaProgress * progress, const gchar * text,
 static void
 balsa_progress_set_fraction(LibBalsaProgress * progress, gdouble fraction)
 {
-    if (*progress == LIBBALSA_PROGRESS_NO
-        || (fraction != 0 && fraction != 1
-            && fraction < prev_fraction + LIBBALSA_PROGRESS_MIN_UPDATE))
+    GTimeVal time_val;
+    guint elapsed;
+
+    if (*progress == LIBBALSA_PROGRESS_NO)
         return;
-    prev_fraction = fraction;
+
+    if (fraction > 0.0 && fraction < min_fraction)
+        return;
+
+    g_get_current_time(&time_val);
+    elapsed = time_val.tv_sec - prev_time_val.tv_sec;
+    elapsed *= G_USEC_PER_SEC;
+    elapsed += time_val.tv_usec - prev_time_val.tv_usec;
+    if (elapsed < LIBBALSA_PROGRESS_MIN_UPDATE_USECS)
+        return;
+
+    g_time_val_add(&time_val, LIBBALSA_PROGRESS_MIN_UPDATE_USECS);
+    min_fraction += LIBBALSA_PROGRESS_MIN_UPDATE_STEP;
 
     gdk_threads_enter();
     if (balsa_app.main_window)
@@ -645,6 +683,19 @@ balsa_progress_set_fraction(LibBalsaProgress * progress, gdouble fraction)
     gdk_threads_leave();
 }
 
+static void
+balsa_progress_set_activity(gboolean set, const gchar * text)
+{
+    gdk_threads_enter();
+    if (balsa_app.main_window) {
+        if (set)
+            balsa_window_increase_activity(balsa_app.main_window, text);
+        else
+            balsa_window_decrease_activity(balsa_app.main_window, text);
+    }
+    gdk_threads_leave();
+}
+
 #if defined(ENABLE_NLS) && defined(HAVE_GPGME)
 static const gchar *
 get_utf8_locale(int category)
@@ -689,6 +740,11 @@ main(int argc, char *argv[])
     threads_init();
 #endif
 
+#ifdef HAVE_RUBRICA
+    /* initialise libxml */
+    LIBXML_TEST_VERSION
+#endif
+
 #ifdef HAVE_GPGME
     /* initialise the gpgme library */
     g_message("init gpgme version %s", gpgme_check_version(NULL));
@@ -732,11 +788,10 @@ main(int argc, char *argv[])
 
     libbalsa_progress_set_text     = balsa_progress_set_text;
     libbalsa_progress_set_fraction = balsa_progress_set_fraction;
+    libbalsa_progress_set_activity = balsa_progress_set_activity;
     
     /* checking for valid config files */
     config_init(cmd_get_stats);
-    libbalsa_address_entry_set_address_book_list(balsa_app.
-                                                 address_book_list);
 
     libbalsa_mailbox_view_table =
 	g_hash_table_new_full(g_str_hash, g_str_equal,
@@ -793,7 +848,7 @@ main(int argc, char *argv[])
         BalsaSendmsg *snd;
         GSList *lst;
         gdk_threads_enter();
-        snd = sendmsg_window_compose(window);
+        snd = sendmsg_window_compose();
         gdk_threads_leave();
         if(opt_compose_email) {
             if(g_ascii_strncasecmp(opt_compose_email, "mailto:", 7) == 0)
@@ -814,10 +869,12 @@ main(int argc, char *argv[])
     g_idle_add((GSourceFunc) scan_mailboxes_idle_cb, NULL);
     g_timeout_add(1801*1000, (GSourceFunc) periodic_expunge_cb, NULL);
 
+    accel_map_load();
     gdk_threads_enter();
     gtk_main();
     gdk_threads_leave();
-    
+    accel_map_save();
+
 #ifdef BALSA_USE_THREADS
     threads_destroy();
 #endif
