@@ -35,7 +35,6 @@
 #include <libgnomeprint/gnome-font.h>
 
 #include <libgnomeprintui/gnome-font-dialog.h>
-#ifdef HAVE_LGPRINT22
 #  include <libgnomeprint/gnome-print-job.h>
 #  include <libgnomeprintui/gnome-print-job-preview.h>
 #  define BALSA_GNOME_PRINT_UI GnomePrintJob
@@ -47,19 +46,6 @@
 #  define BALSA_GNOME_PRINT_UI_CLOSE gnome_print_job_close
 #  define BALSA_GNOME_PRINT_UI_PREVIEW_NEW gnome_print_job_preview_new
 #  define BALSA_GNOME_PRINT_UI_PRINT gnome_print_job_print
-#else
-#  include <libgnomeprint/gnome-print-master.h>
-#  include <libgnomeprintui/gnome-print-master-preview.h>
-#  define BALSA_GNOME_PRINT_UI GnomePrintMaster
-#  define BALSA_GNOME_PRINT_UI_GET_CONFIG gnome_print_master_get_config
-#  define BALSA_GNOME_PRINT_UI_GET_PAGE_SIZE_FROM_CONFIG gnome_print_master_get_page_size_from_config
-#  define BALSA_GNOME_PRINT_UI_GET_CONTEXT gnome_print_master_get_context
-#  define BALSA_GNOME_PRINT_DIALOG_NEW gnome_print_dialog_new_from_master
-#  define BALSA_GNOME_PRINT_UI_NEW gnome_print_master_new()
-#  define BALSA_GNOME_PRINT_UI_CLOSE gnome_print_master_close
-#  define BALSA_GNOME_PRINT_UI_PREVIEW_NEW gnome_print_master_preview_new
-#  define BALSA_GNOME_PRINT_UI_PRINT gnome_print_master_print
-#endif
 
 #include <libgnomeprint/gnome-print-paper.h>
 #include <libgnomeprintui/gnome-print-dialog.h>
@@ -75,6 +61,7 @@
 #include "quote-color.h"
 
 #include <string.h>
+#include <glib/gi18n.h>
 
 #define BALSA_PRINT_TYPE_HEADER     1
 #define BALSA_PRINT_TYPE_SEPARATOR  2
@@ -119,7 +106,7 @@ typedef struct _FontInfo FontInfo;
 typedef struct _CommonInfo CommonInfo;
 
 struct _FontInfo {
-    gchar **font_name;
+    guchar **font_name;
     GnomeFont *font;
     GtkWidget* font_status, *name_label;
     CommonInfo *common_info;
@@ -229,7 +216,7 @@ print_foot_lines(PrintInfo * pi, GnomeFont * font, float y,
 	gnome_print_moveto(pi->pc, 
 			   pi->margin_left + (pi->printable_width - width) / 2.0,
 			   y);
-	gnome_print_show(pi->pc, ptr);
+	gnome_print_show(pi->pc, (guchar *) ptr);
 	ptr = eol;
 	if (eol) {
 	    *eol = '\n';
@@ -252,7 +239,7 @@ start_new_page_real(PrintInfo * pi)
     if (balsa_app.debug)
 	g_print("Processing page %s\n", buf);
 
-    gnome_print_beginpage(pi->pc, buf);
+    gnome_print_beginpage(pi->pc, (guchar *) buf);
     /* print the page number */
     if (balsa_app.print_highlight_cited)
 	gnome_print_setrgbcolor (pi->pc, 0.0, 0.0, 0.0);
@@ -262,7 +249,7 @@ start_new_page_real(PrintInfo * pi)
     width = gnome_font_get_width_utf8(pi->header_font, page_no);
     gnome_print_moveto(pi->pc, pi->page_width - pi->margin_left - width,
 		       ypos);
-    gnome_print_show(pi->pc, page_no);
+    gnome_print_show(pi->pc, (guchar *) page_no);
     g_free(page_no);
     
     /* print the footer */
@@ -316,12 +303,14 @@ print_header_string(GList **header_list, const gchar *field_id,
     hdr_pair = g_new0(gchar *, 3);
     hdr_pair[0] = g_strdup(label);
     hdr_pair[1] = g_strdup(value);
+    libbalsa_utf8_sanitize(&hdr_pair[1], balsa_app.convert_unknown_8bit,
+			   NULL);
     *header_list = g_list_append(*header_list, hdr_pair);
 }
 
 static void
 print_header_list(GList **header_list, const gchar *field_id,
-		  const gchar *label, GList *values)
+		  const gchar *label, InternetAddressList * values)
 {
     gchar **hdr_pair;
 
@@ -333,7 +322,9 @@ print_header_list(GList **header_list, const gchar *field_id,
 
     hdr_pair = g_new0(gchar *, 3);
     hdr_pair[0] = g_strdup(label);
-    hdr_pair[1] = libbalsa_make_string_from_list(values);
+    hdr_pair[1] = internet_address_list_to_string(values, FALSE);
+    libbalsa_utf8_sanitize(&hdr_pair[1], balsa_app.convert_unknown_8bit,
+			   NULL);
     *header_list = g_list_append(*header_list, hdr_pair);
 }
 
@@ -361,12 +352,12 @@ prepare_header_real(PrintInfo * pi, LibBalsaMessageBody * sig_body,
     }
     g_free(subject);
 
-    date = libbalsa_message_headers_date_to_gchar(headers, balsa_app.date_string);
+    date = libbalsa_message_headers_date_to_utf8(headers, balsa_app.date_string);
     print_header_string (&pdata->headers, "date", _("Date:"), date);
     g_free(date);
 
     if (headers->from) {
-	gchar *from = libbalsa_address_to_gchar(headers->from, 0);
+	gchar *from = internet_address_list_to_string(headers->from, FALSE);
 	print_header_string (&pdata->headers, "from", _("From:"), from);
 	g_free(from);
     }
@@ -377,7 +368,8 @@ prepare_header_real(PrintInfo * pi, LibBalsaMessageBody * sig_body,
     print_header_string (&pdata->headers, "fcc", _("Fcc:"), headers->fcc_url);
 
     if (headers->dispnotify_to) {
-	gchar *mdn_to = libbalsa_address_to_gchar(headers->dispnotify_to, 0);
+	gchar *mdn_to =
+	    internet_address_list_to_string(headers->dispnotify_to, FALSE);
 	print_header_string (&pdata->headers, "disposition-notification-to", 
 			     _("Disposition-Notification-To:"), mdn_to);
 	g_free(mdn_to);
@@ -481,7 +473,7 @@ prepare_message_header(PrintInfo * pi, LibBalsaMessageBody * body)
 	footer_string = g_string_new(subject);
     g_free(subject);
 
-    date = libbalsa_message_date_to_gchar(pi->message, balsa_app.date_string);
+    date = libbalsa_message_date_to_utf8(pi->message, balsa_app.date_string);
     if (footer_string) {
 	footer_string = g_string_append(footer_string, " - ");
 	footer_string = g_string_append(footer_string, date);
@@ -491,7 +483,9 @@ prepare_message_header(PrintInfo * pi, LibBalsaMessageBody * body)
     g_free(date);
 
     if (pi->message->headers->from) {
-	gchar *from = libbalsa_address_to_gchar(pi->message->headers->from, 0);
+	gchar *from =
+	    internet_address_list_to_string(pi->message->headers->from,
+		                            FALSE);
 	if (footer_string) {
 	    footer_string = g_string_prepend(footer_string, " - ");
 	    footer_string = g_string_prepend(footer_string, from);
@@ -531,7 +525,7 @@ print_header_val(PrintInfo * pi, gint x, float * y,
 	if (eol)
 	    *eol = '\0';
 	gnome_print_moveto(pi->pc, x, *y);
-	gnome_print_show(pi->pc, ptr);
+	gnome_print_show(pi->pc, (guchar *) ptr);
 	ptr = eol;
 	if (eol)
 	    ptr++;
@@ -565,7 +559,7 @@ print_header(PrintInfo * pi, gpointer * data)
 	if (pi->ypos < pi->margin_bottom)
 	    start_new_page(pi);
 	gnome_print_moveto(pi->pc, pi->margin_left, pi->ypos);
-	gnome_print_show(pi->pc, pair[0]);
+	gnome_print_show(pi->pc, (guchar *) pair[0]);
 	print_header_val(pi, pi->margin_left + pdata->header_label_width,
 			 &pi->ypos, font_size, pair[1], pi->header_font);
 	g_strfreev(pair);
@@ -651,6 +645,7 @@ prepare_html(PrintInfo * pi, LibBalsaMessageBody * body, gpointer data)
     gchar *html_text;
     gchar *conttype;
     LibBalsaHTMLType html_type;
+    GError *err = NULL;
 
     conttype = libbalsa_message_body_get_mime_type(body);
     html_type = libbalsa_html_type(conttype);
@@ -674,16 +669,23 @@ prepare_html(PrintInfo * pi, LibBalsaMessageBody * body, gpointer data)
 	return;
     }
 
-    len = libbalsa_message_body_get_content(body, &html_text);
-    if (!html_text)
+    len = libbalsa_message_body_get_content(body, &html_text, &err);
+    if (!html_text) {
+        balsa_information(LIBBALSA_INFORMATION_ERROR,
+                          _("Could not get a part: %s"),
+                          err ? err->message : "Unknown error");
+        g_clear_error(&err);
 	return;
+    }
 
     len = libbalsa_html_filter(html_type, &html_text, len);
 
     pdata = g_new(HtmlInfo, 1);
     pdata->id_tag = BALSA_PRINT_TYPE_HTML;
     pdata->html =
-	libbalsa_html_new(html_text, len, pi->message, NULL);
+	libbalsa_html_new(html_text, len,
+			  libbalsa_message_body_charset(body),
+			  pi->message, NULL);
     g_free(html_text);
 
     if (libbalsa_html_can_zoom(pdata->html)) {
@@ -729,7 +731,7 @@ print_html_header(GtkWidget * html, GnomePrintContext * print_context,
     gnome_print_moveto(pi->pc,
 		       pi->page_width - pi->margin_left - page_no_width,
 		       ypos);
-    gnome_print_show(pi->pc, page_no);
+    gnome_print_show(pi->pc, (guchar *) page_no);
     g_free(page_no);
 }
 
@@ -871,7 +873,7 @@ prepare_plaintext(PrintInfo * pi, LibBalsaMessageBody * body, gpointer data)
     if (body->buffer)
 	textbuf = g_strdup(body->buffer);
     else
-	libbalsa_message_body_get_content(body, &textbuf);
+	libbalsa_message_body_get_content(body, &textbuf, NULL);
 
     /* fake an empty buffer if textbuf is NULL */
     if (!textbuf)
@@ -938,7 +940,7 @@ print_plaintext(PrintInfo * pi, gpointer * data)
  		gnome_print_setrgbcolor (pi->pc, 0.0, 0.0, 0.0);
  	}
  	gnome_print_moveto(pi->pc, pi->margin_left, pi->ypos);
- 	gnome_print_show(pi->pc, lineInfo->lineData);
+ 	gnome_print_show(pi->pc, (guchar *) lineInfo->lineData);
  	g_free(lineInfo->lineData);
  	g_free(l->data);
  	l = l->next;
@@ -960,9 +962,8 @@ static void
 prepare_default(PrintInfo * pi, LibBalsaMessageBody * body, gpointer data)
 {
     DefaultInfo *pdata;
-    gchar *icon_name, *conttype;
+    gchar *conttype;
     gint hdr = 0, lines;
-    GError* err = NULL;
 
     pdata = g_malloc(sizeof(DefaultInfo));
     pdata->id_tag = BALSA_PRINT_TYPE_DEFAULT;
@@ -970,12 +971,10 @@ prepare_default(PrintInfo * pi, LibBalsaMessageBody * body, gpointer data)
     conttype = libbalsa_message_body_get_mime_type(body);
 
     /* get a pixbuf according to the mime type */
-    icon_name = libbalsa_icon_finder(conttype, NULL, NULL);
-    pdata->pixbuf = gdk_pixbuf_new_from_file(icon_name, &err);
-    if(err) { g_warning("error loading pixbuf."); g_error_free(err); }
+    pdata->pixbuf = libbalsa_icon_finder(conttype, NULL, NULL,
+					 GTK_ICON_SIZE_DND);
     pdata->image_width = gdk_pixbuf_get_width (pdata->pixbuf);
     pdata->image_height = gdk_pixbuf_get_height (pdata->pixbuf);
-    g_free(icon_name);
 
     /* gather some info about this part */
     pdata->labels = g_new0(gchar *, 5); /* four fields, one terminator */
@@ -1034,10 +1033,10 @@ print_image_from_pixbuf(GnomePrintContext * gpc, GdkPixbuf * pixbuf)
     width     = gdk_pixbuf_get_width(pixbuf);
 
     if (has_alpha)
-        gnome_print_rgbaimage(gpc, (char *) raw_image, width, height,
+        gnome_print_rgbaimage(gpc, (guchar *) raw_image, width, height,
                               rowstride);
     else
-        gnome_print_rgbimage(gpc, (char *) raw_image, width, height,
+        gnome_print_rgbimage(gpc, (guchar *) raw_image, width, height,
                              rowstride);
 }
 
@@ -1075,7 +1074,7 @@ print_default(PrintInfo * pi, gpointer data)
     offset = pi->margin_left + pdata->image_width + 10;
     for (i = 0; pdata->labels[i]; i += 2) {
 	gnome_print_moveto(pi->pc, offset, pi->ypos);
-	gnome_print_show(pi->pc, pdata->labels[i]);
+	gnome_print_show(pi->pc, (guchar *) pdata->labels[i]);
 	print_header_val(pi, offset + pdata->label_width, &pi->ypos,
  			 font_size, pdata->labels[i + 1], pi->header_font);
  	pi->ypos -= font_size;
@@ -1235,7 +1234,7 @@ print_crypto_signature(PrintInfo * pi, gpointer * data)
 	    gnome_print_setfont(pi->pc, pi->header_font);
 	}
  	gnome_print_moveto(pi->pc, pi->margin_left, pi->ypos);
- 	gnome_print_show(pi->pc, lineInfo->lineData);
+ 	gnome_print_show(pi->pc, (guchar *) lineInfo->lineData);
  	g_free(lineInfo->lineData);
  	g_free(l->data);
  	l = l->next;
@@ -1333,11 +1332,11 @@ find_font(const gchar * name)
     space = strrchr(copy, ' ');
     if (space)
         *space = 0;
-    face = gnome_font_face_find(copy);
+    face = gnome_font_face_find((guchar *) copy);
     g_free(copy);
     if (face) {
         gnome_font_face_unref(face);
-        font = gnome_font_find_from_full_name(name);
+        font = gnome_font_find_from_full_name((guchar *) name);
     }
     return font;
 #endif          /*  GNOME_FONT_FIND_HANDLES_BAD_NAME_SANELY */
@@ -1349,7 +1348,7 @@ get_length_from_config(GnomePrintConfig * config, const gchar * key)
     const GnomePrintUnit *unit;
     gdouble length = 0.0;
 
-    if (gnome_print_config_get_length(config, key, &length, &unit))
+    if (gnome_print_config_get_length(config, (guchar *) key, &length, &unit))
         gnome_print_convert_distance(&length, unit, GNOME_PRINT_PS_UNIT);
 
     return length;
@@ -1542,14 +1541,14 @@ set_dialog_buttons_sensitive(CommonInfo * ci)
 static void
 set_font_status(FontInfo *fi)
 {
-    gtk_label_set_text(GTK_LABEL(fi->name_label), *fi->font_name);
+    gtk_label_set_text(GTK_LABEL(fi->name_label), (gchar *) *fi->font_name);
     if(fi->font) 
 	gtk_label_set_text(GTK_LABEL(fi->font_status),
 			   _("Font available for printing"));
     else {
 	GnomeFont* fncl = 
 	    gnome_font_find_closest_from_full_name(*fi->font_name);
-	gchar* fn = gnome_font_get_full_name(fncl);
+	guchar* fn = gnome_font_get_full_name(fncl);
 	gchar *msg = 
 	    g_strdup_printf(_("Font <b>not</b> available for printing. "
 			      "Closest: %s"), fn);
@@ -1599,7 +1598,7 @@ font_frame(gchar * title, FontInfo * fi)
     GtkWidget *hbox    = gtk_hbox_new(FALSE, 3);
     GtkWidget *button = gtk_button_new_with_label(_("Change..."));
 
-    fi->name_label = gtk_label_new(*fi->font_name);
+    fi->name_label = gtk_label_new((gchar *) *fi->font_name);
     gtk_box_pack_start(GTK_BOX(hbox), fi->name_label, TRUE, TRUE, 5);
     gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 5);
     g_signal_connect(G_OBJECT(button), "clicked", 
@@ -1631,8 +1630,9 @@ print_dialog(CommonInfo * ci)
     GtkWidget  *chkbut;
     GList      *childList;
 
-    dialog = BALSA_GNOME_PRINT_DIALOG_NEW(ci->master, _("Print message"),
-				    GNOME_PRINT_DIALOG_COPIES);
+    dialog = BALSA_GNOME_PRINT_DIALOG_NEW(ci->master,
+                                          (guchar *) _("Print message"),
+                                          GNOME_PRINT_DIALOG_COPIES);
     ci->dialog = dialog;
     common_info_setup(ci);
     gtk_window_set_wmclass(GTK_WINDOW(dialog), "print", "Balsa");
@@ -1677,7 +1677,7 @@ print_dialog(CommonInfo * ci)
 static void
 font_info_setup(FontInfo * fi, gchar ** font_name, CommonInfo * ci)
 {
-    fi->font_name = font_name;
+    fi->font_name = (guchar **) font_name;
     fi->common_info = ci;
     fi->font = find_font(*font_name);
     if (!fi->font)
@@ -1758,25 +1758,20 @@ print_response_cb(GtkDialog * dialog, gint response, CommonInfo * ci)
     }
 
     config = BALSA_GNOME_PRINT_UI_GET_CONFIG(ci->master);
+#define CONFIG_GET(key) \
+    ((gchar *) gnome_print_config_get(config, (guchar *) (key)))
     g_free(balsa_app.paper_size);
-    balsa_app.paper_size =
-        gnome_print_config_get(config, GNOME_PRINT_KEY_PAPER_SIZE); 
-    balsa_app.print_unit =
-        gnome_print_config_get(config, GNOME_PRINT_KEY_PREFERED_UNIT); 
-    balsa_app.margin_left =
-        gnome_print_config_get(config, GNOME_PRINT_KEY_PAGE_MARGIN_LEFT); 
-    balsa_app.margin_top =
-        gnome_print_config_get(config, GNOME_PRINT_KEY_PAGE_MARGIN_TOP); 
-    balsa_app.margin_right =
-        gnome_print_config_get(config, GNOME_PRINT_KEY_PAGE_MARGIN_RIGHT);
-    balsa_app.margin_bottom =
-        gnome_print_config_get(config, GNOME_PRINT_KEY_PAGE_MARGIN_BOTTOM); 
-    balsa_app.print_layout =
-        gnome_print_config_get(config, GNOME_PRINT_KEY_LAYOUT); 
+    balsa_app.paper_size = CONFIG_GET(GNOME_PRINT_KEY_PAPER_SIZE); 
+    balsa_app.print_unit = CONFIG_GET(GNOME_PRINT_KEY_PREFERED_UNIT); 
+    balsa_app.margin_left = CONFIG_GET(GNOME_PRINT_KEY_PAGE_MARGIN_LEFT); 
+    balsa_app.margin_top = CONFIG_GET(GNOME_PRINT_KEY_PAGE_MARGIN_TOP); 
+    balsa_app.margin_right = CONFIG_GET(GNOME_PRINT_KEY_PAGE_MARGIN_RIGHT);
+    balsa_app.margin_bottom = CONFIG_GET(GNOME_PRINT_KEY_PAGE_MARGIN_BOTTOM); 
+    balsa_app.print_layout = CONFIG_GET(GNOME_PRINT_KEY_LAYOUT); 
     balsa_app.paper_orientation =
-        gnome_print_config_get(config, GNOME_PRINT_KEY_PAPER_ORIENTATION); 
-    balsa_app.page_orientation =
-        gnome_print_config_get(config, GNOME_PRINT_KEY_PAGE_ORIENTATION); 
+	CONFIG_GET(GNOME_PRINT_KEY_PAPER_ORIENTATION); 
+    balsa_app.page_orientation = CONFIG_GET(GNOME_PRINT_KEY_PAGE_ORIENTATION); 
+#undef CONFIG_GET
 
     gnome_print_config_unref(config);
 
@@ -1788,7 +1783,8 @@ print_response_cb(GtkDialog * dialog, gint response, CommonInfo * ci)
     if (preview) {
 	GtkWidget *preview_widget =
 	    BALSA_GNOME_PRINT_UI_PREVIEW_NEW(ci->master,
-		 			   _("Balsa: message print preview"));
+                                             (guchar *)
+					     _("Balsa: message print preview"));
         gtk_window_set_wmclass(GTK_WINDOW(preview_widget), "print-preview",
                                "Balsa");
 	gtk_widget_show(preview_widget);
@@ -1853,33 +1849,30 @@ message_print(LibBalsaMessage * msg, GtkWindow * parent)
      * initial value in the Paper page. Is there some Gnome-2-wide
      * repository for data like this? */
     config = BALSA_GNOME_PRINT_UI_GET_CONFIG(ci->master);
-    gnome_print_config_set(config, GNOME_PRINT_KEY_PAPER_SIZE, 
-                           balsa_app.paper_size);
+#define CONFIG_SET(key, value) \
+    gnome_print_config_set(config, (guchar *) (key), (guchar *) (value))
+    CONFIG_SET(GNOME_PRINT_KEY_PAPER_SIZE, balsa_app.paper_size);
     if(balsa_app.print_unit)
-	gnome_print_config_set(config, GNOME_PRINT_KEY_PREFERED_UNIT,
-			       balsa_app.print_unit); 
+	CONFIG_SET(GNOME_PRINT_KEY_PREFERED_UNIT, balsa_app.print_unit); 
     if(balsa_app.margin_left)
-	gnome_print_config_set(config, GNOME_PRINT_KEY_PAGE_MARGIN_LEFT,
-			       balsa_app.margin_left); 
+	CONFIG_SET(GNOME_PRINT_KEY_PAGE_MARGIN_LEFT, balsa_app.margin_left); 
     if(balsa_app.margin_top)
-	gnome_print_config_set(config, GNOME_PRINT_KEY_PAGE_MARGIN_TOP,
-			       balsa_app.margin_top); 
+	CONFIG_SET(GNOME_PRINT_KEY_PAGE_MARGIN_TOP, balsa_app.margin_top); 
     if(balsa_app.margin_right)
-	gnome_print_config_set(config, GNOME_PRINT_KEY_PAGE_MARGIN_RIGHT,
-			       balsa_app.margin_right);
+	CONFIG_SET(GNOME_PRINT_KEY_PAGE_MARGIN_RIGHT, balsa_app.margin_right);
     if(balsa_app.margin_bottom)
-	gnome_print_config_set(config, GNOME_PRINT_KEY_PAGE_MARGIN_BOTTOM,
-			       balsa_app.margin_bottom); 
+	CONFIG_SET(GNOME_PRINT_KEY_PAGE_MARGIN_BOTTOM,
+		   balsa_app.margin_bottom); 
     if(balsa_app.print_layout)
-	gnome_print_config_set(config, GNOME_PRINT_KEY_LAYOUT,
-			       balsa_app.print_layout); 
+	CONFIG_SET(GNOME_PRINT_KEY_LAYOUT, balsa_app.print_layout); 
     if(balsa_app.paper_orientation)
-	gnome_print_config_set(config, GNOME_PRINT_KEY_PAPER_ORIENTATION,
-			       balsa_app.paper_orientation); 
+	CONFIG_SET(GNOME_PRINT_KEY_PAPER_ORIENTATION,
+		   balsa_app.paper_orientation); 
     if(balsa_app.page_orientation)
-	gnome_print_config_set(config, GNOME_PRINT_KEY_PAPER_ORIENTATION,
-			       balsa_app.page_orientation); 
+	CONFIG_SET(GNOME_PRINT_KEY_PAPER_ORIENTATION,
+		   balsa_app.page_orientation); 
     gnome_print_config_unref(config);
+#undef CONFIG_SET
     
     ci->dialog = print_dialog(ci);
     gtk_window_set_transient_for(GTK_WINDOW(ci->dialog), parent);
