@@ -25,7 +25,7 @@
 #include <string.h>
 #include <gconf/gconf-client.h>
 #include <gnome.h>
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "balsa-app.h"
 #include "save-restore.h"
 #include "server.h"
@@ -126,121 +126,23 @@ d_get_gint(const gchar * key, gint def_val)
 /* save/load_toolbars:
    handle customized toolbars for main/message preview and compose windows.
 */
-static struct {
-    const gchar *key;
-    GSList **current;
-} toolbars[] = {
-    { "toolbar-MainWindow",    &balsa_app.main_window_toolbar_current },
-    { "toolbar-ComposeWindow", &balsa_app.compose_window_toolbar_current },
-    { "toolbar-MessageWindow", &balsa_app.message_window_toolbar_current }
-};
 
 static void
 save_toolbars(void)
 {
-    guint i, j;
-
     libbalsa_conf_remove_group("Toolbars");
     libbalsa_conf_push_group("Toolbars");
     libbalsa_conf_set_int("WrapButtonText",
                          balsa_app.toolbar_wrap_button_text);
     libbalsa_conf_pop_group();
-
-    for (i = 0; i < ELEMENTS(toolbars); i++) {
-        GSList *list;
-
-        libbalsa_conf_remove_group(toolbars[i].key);
-        libbalsa_conf_push_group(toolbars[i].key);
-
-        for (j = 0, list = *toolbars[i].current; list;
-             j++, list = list->next) {
-            gchar *key = g_strdup_printf("Item%d", j);
-            libbalsa_conf_set_string(key, list->data);
-            g_free(key);
-        }
-        libbalsa_conf_pop_group();
-    }
 }
 
 static void
 load_toolbars(void)
 {
-    guint i, j, items;
-    char tmpkey[256];
-    gchar *key;
-    GSList **list;
-    guint changed = 0;
-
-    tmpkey[sizeof tmpkey - 1] = '\0';
-
     libbalsa_conf_push_group("Toolbars");
     balsa_app.toolbar_wrap_button_text = d_get_gint("WrapButtonText", 1);
     libbalsa_conf_pop_group();
-
-    items = 0;
-    for (i = 0; i < ELEMENTS(toolbars); i++) {
-        libbalsa_conf_push_group(toolbars[i].key);
-
-        list = toolbars[i].current;
-        for (j = 0;; j++) {
-            gchar *item;
-	    const gchar *real_item;
-
-            key = g_strdup_printf("Item%d", j);
-            item = libbalsa_conf_get_string(key);
-            g_free(key);
-
-            if (!item)
-                break;
-	    if ((real_item = balsa_toolbar_sanitize_id(item))) {
-		*list = g_slist_append(*list, g_strdup(real_item));
-		items++;
-		if (strcmp(real_item, item) != 0)
-		    ++changed;
-	    }
-	    g_free(item);
-        }
-        libbalsa_conf_pop_group();
-    }
-
-    if (items)
-        return;
-
-    /* We didn't find new-style toolbar configs, so we'll try the old
-     * style */
-    libbalsa_conf_push_group("Toolbars");
-    for (i = 0; i < ELEMENTS(toolbars); i++) {
-        guint type;
-
-        snprintf(tmpkey, sizeof tmpkey - 1, "Toolbar%dID", i);
-        type = d_get_gint(tmpkey, -1);
-
-        if (type >= ELEMENTS(toolbars)) {
-            continue;
-        }
-
-        snprintf(tmpkey, sizeof tmpkey - 1, "Toolbar%dItemCount", i);
-        items = d_get_gint(tmpkey, 0);
-
-        list = toolbars[type].current;
-        for (j = 0; j < items; j++) {
-            gchar *item;
-	    const gchar *real_item;
-
-            snprintf(tmpkey, sizeof tmpkey - 1, "Toolbar%dItem%d", i, j);
-            item = libbalsa_conf_get_string(tmpkey);
- 	    if ((real_item = balsa_toolbar_sanitize_id(item))) {
- 		*list = g_slist_append(*list, g_strdup(real_item));
-		if (strcmp(real_item, item) != 0)
-		    ++changed;
-	    }
- 	    g_free(item);
-        }
-    }
-    libbalsa_conf_pop_group();
-
-    if (changed)
-	save_toolbars();
 }
 
 /* config_mailbox_set_as_special:
@@ -573,7 +475,7 @@ static gboolean
 config_warning_idle(const gchar * text)
 {
     gdk_threads_enter();
-    balsa_information(LIBBALSA_INFORMATION_WARNING, text);
+    balsa_information(LIBBALSA_INFORMATION_WARNING, "%s", text);
     gdk_threads_leave();
     return FALSE;
 }
@@ -668,6 +570,7 @@ config_global_load(void)
 {
     gboolean def_used;
     guint tmp;
+    static gboolean new_user = FALSE;
 
     config_address_books_load();
 #if ENABLE_ESMTP
@@ -735,14 +638,21 @@ config_global_load(void)
     /* ... window sizes */
     balsa_app.mw_width = libbalsa_conf_get_int("MainWindowWidth=640");
     balsa_app.mw_height = libbalsa_conf_get_int("MainWindowHeight=480");
+    balsa_app.mw_maximized =
+        libbalsa_conf_get_bool("MainWindowMaximized=false");
     balsa_app.mblist_width = libbalsa_conf_get_int("MailboxListWidth=130");
     /* sendmsg window sizes */
     balsa_app.sw_width = libbalsa_conf_get_int("SendMsgWindowWidth=640");
     balsa_app.sw_height = libbalsa_conf_get_int("SendMsgWindowHeight=480");
+    balsa_app.sw_maximized =
+        libbalsa_conf_get_bool("SendmsgWindowMaximized=false");
+    /* message window sizes */
     balsa_app.message_window_width =
         libbalsa_conf_get_int("MessageWindowWidth=400");
     balsa_app.message_window_height =
         libbalsa_conf_get_int("MessageWindowHeight=500");
+    balsa_app.message_window_maximized =
+        libbalsa_conf_get_bool("MessageWindowMaximized=false");
     /* FIXME: PKGW: why comment this out? Breaks my Transfer context menu. */
     if (balsa_app.mblist_width < 100)
 	balsa_app.mblist_width = 170;
@@ -965,6 +875,10 @@ config_global_load(void)
 #if HAVE_GTKSPELL
     balsa_app.spell_check_lang =
         libbalsa_conf_get_string("SpellCheckLanguage");
+    balsa_app.spell_check_active =
+        libbalsa_conf_get_bool_with_default("SpellCheckActive", &def_used);
+    if (def_used)
+        balsa_app.spell_check_active = balsa_app.spell_check_lang != NULL;
 #else                           /* HAVE_GTKSPELL */
     balsa_app.module = d_get_gint("PspellModule", DEFAULT_PSPELL_MODULE);
     balsa_app.suggestion_mode =
@@ -1105,7 +1019,7 @@ config_global_load(void)
     balsa_app.quote_str = libbalsa_conf_get_string("QuoteString=> ");
     g_free(balsa_app.compose_headers);
     balsa_app.compose_headers =
-	libbalsa_conf_get_string("ComposeHeaders=to subject cc");
+	libbalsa_conf_get_string("ComposeHeaders=Recipients Subject");
 
     /* Obsolete. */
     libbalsa_conf_get_bool_with_default("RequestDispositionNotification=false",
@@ -1127,6 +1041,7 @@ config_global_load(void)
 
     if(!balsa_app.local_mail_directory) {
 	libbalsa_conf_pop_group();
+        new_user = TRUE;
 	return FALSE;
     }
     balsa_app.root_node =
@@ -1186,6 +1101,31 @@ config_global_load(void)
      * --without-esmtp build). */
     config_identities_load();
 
+    if (!new_user) {
+        /* Notify user about any changes */
+        libbalsa_conf_push_group("Notifications");
+        if (!libbalsa_conf_get_bool("GtkUIManager")) {
+            g_idle_add((GSourceFunc) config_warning_idle,
+                       _("This version of Balsa uses a new user interface; "
+                         "if you have changed Balsa's keyboard accelerators, "
+                         "you will need to set them again."));
+            libbalsa_conf_set_bool("GtkUIManager", TRUE);
+        }
+        if (!libbalsa_conf_get_bool("LibBalsaAddressView")) {
+            /* No warning */
+            if (!libbalsa_find_word("Recipients",
+                                    balsa_app.compose_headers)) {
+                gchar *compose_headers =
+                    g_strconcat(balsa_app.compose_headers, " Recipients",
+                                NULL);
+                g_free(balsa_app.compose_headers);
+                balsa_app.compose_headers = compose_headers;
+            }
+            libbalsa_conf_set_bool("LibBalsaAddressView", TRUE);
+        }
+        libbalsa_conf_pop_group();
+    }
+
     return TRUE;
 }				/* config_global_load */
 
@@ -1278,15 +1218,25 @@ config_save(void)
 			 balsa_app.index_subject_width);
     libbalsa_conf_set_int("IndexDateWidth", balsa_app.index_date_width);
     libbalsa_conf_set_int("IndexSizeWidth", balsa_app.index_size_width);
+
     libbalsa_conf_set_int("MainWindowWidth", balsa_app.mw_width);
     libbalsa_conf_set_int("MainWindowHeight", balsa_app.mw_height);
+    libbalsa_conf_set_bool("MainWindowMaximized",
+                           !!balsa_app.mw_maximized);
     libbalsa_conf_set_int("MailboxListWidth", balsa_app.mblist_width);
+
     libbalsa_conf_set_int("SendMsgWindowWidth", balsa_app.sw_width);
     libbalsa_conf_set_int("SendMsgWindowHeight", balsa_app.sw_height);
+    libbalsa_conf_set_bool("SendmsgWindowMaximized",
+                           !!balsa_app.sw_maximized);
+
     libbalsa_conf_set_int("MessageWindowWidth",
                          balsa_app.message_window_width);
     libbalsa_conf_set_int("MessageWindowHeight",
                          balsa_app.message_window_height);
+    libbalsa_conf_set_bool("MessageWindowMaximized",
+                           !!balsa_app.message_window_maximized);
+
     libbalsa_conf_set_int("NotebookHeight", balsa_app.notebook_height);
 
     libbalsa_conf_pop_group();
@@ -1417,9 +1367,10 @@ config_save(void)
     libbalsa_conf_push_group("Spelling");
 
 #if HAVE_GTKSPELL
-    if (balsa_app.spell_check_lang)
-        libbalsa_conf_set_string("SpellCheckLanguage",
-                                 balsa_app.spell_check_lang);
+    libbalsa_conf_set_string("SpellCheckLanguage",
+                             balsa_app.spell_check_lang);
+    libbalsa_conf_set_bool("SpellCheckActive", 
+                           balsa_app.spell_check_active);
 #else                           /* HAVE_GTKSPELL */
     libbalsa_conf_set_int("PspellModule", balsa_app.module);
     libbalsa_conf_set_int("PspellSuggestMode", balsa_app.suggestion_mode);
