@@ -26,6 +26,8 @@
 extern "C" {
 #endif				/* __cplusplus */
 
+#include "address-view.h"
+
     typedef enum {
        SEND_NORMAL,            /* initialized by Compose */
        SEND_REPLY,             /* by Reply               */
@@ -48,49 +50,29 @@ extern "C" {
 #define VIEW_MENU_LENGTH 5
 #endif
     typedef struct _BalsaSendmsg BalsaSendmsg;
-    typedef struct _BalsaSendmsgAddress BalsaSendmsgAddress;
-
-    struct _BalsaSendmsgAddress {
-        BalsaSendmsg *bsmsg;
-        GtkWidget *label;
-        gint min_addresses, max_addresses;
-        gboolean ready;
-    };
 
     struct _BalsaSendmsg {
 	GtkWidget *window;
-	GtkWidget *to[3], *from[3], *subject[2], *cc[3], *bcc[3], *fcc[3];
-        BalsaSendmsgAddress to_info, cc_info, bcc_info;
-#if !defined(ENABLE_TOUCH_UI)        
-        GtkWidget *reply_to[3];
-        BalsaSendmsgAddress reply_to_info;
-#endif
+        LibBalsaAddressView *recipient_view, *replyto_view;
+	GtkWidget *from[2], *recipients[2], *replyto[2], *subject[2], *fcc[2];
 	GtkWidget *attachments[4];
         gchar *in_reply_to;
         GList *references;
 	GtkWidget *text;
 #if !HAVE_GTKSPELL
 	GtkWidget *spell_checker;
-#else
-	GtkWidget *spell_check_menu_item;
-        gboolean   spell_check_error;
 #endif                          /* HAVE_GTKSPELL */
 	GtkWidget *notebook;
 	LibBalsaMessage *parent_message; /* to which we're replying     */
 	LibBalsaMessage *draft_message;  /* where the message was saved */
 	SendType type;
 	/* language selection related data */
-	gchar *charset;
-	GSList *charsets;
 	const gchar *spell_check_lang;
 	GtkWidget *current_language_menu;
 	/* identity related data */
 	LibBalsaIdentity* ident;
         /* fcc mailbox */
         gchar *fcc_url;
-	/* widgets to be disabled when the address is incorrect */
-	GtkWidget *ready_widgets[3];
-	GtkWidget *view_checkitems[VIEW_MENU_LENGTH];
 	gboolean update_config; /* is the window being set up or in normal  */
 	                        /* operation and user actions should update */
 	                        /* the config */
@@ -105,7 +87,6 @@ extern "C" {
         SendmsgState state;
         gulong identities_changed_id;
 	gboolean flow;          /* send format=flowed */ 
-	GtkWidget *flow_widget;
 	gboolean req_dispnotify; /* send a MDN */ 
 	gboolean quit_on_close; /* quit balsa after the compose window */
 	                        /* is closed.                          */
@@ -114,31 +95,34 @@ extern "C" {
         GtkStyle *bad_address_style;  
 #ifdef HAVE_GPGME
 	guint gpg_mode;
-	GtkWidget *gpg_sign_menu_item;
-	GtkWidget *gpg_encrypt_menu_item;
 #endif
         GtkWidget *header_table;
 
 #if !HAVE_GTKSOURCEVIEW
         GtkTextBuffer *buffer2;       /* Undo buffer. */
 #endif                          /* HAVE_GTKSOURCEVIEW */
-        GtkWidget *undo_widget;
-        GtkWidget *redo_widget;
 
         /* To update cursor after text is inserted. */
         GtkTextMark *insert_mark;
+
+        GtkActionGroup *action_group;
+	GtkActionGroup *ready_action_group;
+#if !defined(ENABLE_TOUCH_UI)
+        GtkActionGroup *gpg_action_group;
+#endif                          /* ENABLE_TOUCH_UI */
     };
 
-    BalsaSendmsg *sendmsg_window_compose(GtkWidget *);
-    BalsaSendmsg *sendmsg_window_reply(GtkWidget *, LibBalsaMailbox *,
+    BalsaSendmsg *sendmsg_window_compose(void);
+    BalsaSendmsg *sendmsg_window_compose_with_address(const gchar *
+                                                      address);
+    BalsaSendmsg *sendmsg_window_reply(LibBalsaMailbox *,
                                        guint msgno, SendType rt);
-    BalsaSendmsg *sendmsg_window_reply_embedded(GtkWidget *w,
-                                                LibBalsaMessageBody *part,
+    BalsaSendmsg *sendmsg_window_reply_embedded(LibBalsaMessageBody *part,
                                                 SendType reply_type);
 
-    BalsaSendmsg *sendmsg_window_forward(GtkWidget *, LibBalsaMailbox *,
+    BalsaSendmsg *sendmsg_window_forward(LibBalsaMailbox *,
                                          guint msgno, gboolean attach);
-    BalsaSendmsg *sendmsg_window_continue(GtkWidget *, LibBalsaMailbox *,
+    BalsaSendmsg *sendmsg_window_continue(LibBalsaMailbox *,
                                           guint msgno);
 
     void sendmsg_window_set_field(BalsaSendmsg *bsmsg, const gchar* key,
@@ -152,11 +136,11 @@ extern "C" {
 
     void sendmsg_window_process_url(const char *url, field_setter func,
 				    void *data);
-    BalsaSendmsg *sendmsg_window_new_from_list(GtkWidget * w,
-                                               LibBalsaMailbox * mailbox,
+    BalsaSendmsg *sendmsg_window_new_from_list(LibBalsaMailbox * mailbox,
                                                GArray * selected,
                                                SendType type);
-    BalsaToolbarModel *sendmsg_window_get_toolbar_model(void);
+    BalsaToolbarModel *sendmsg_window_get_toolbar_model(GtkUIManager **
+                                                        ui_manager);
 
 #define SENDMSG_WINDOW_QUIT_ON_CLOSE(bsmsg) ((bsmsg)->quit_on_close=TRUE)
 
